@@ -32,6 +32,7 @@ import java.beans.PropertyChangeEvent;
 import java.beans.PropertyChangeListener;
 import java.net.MalformedURLException;
 import java.net.URL;
+import java.security.AccessControlException;
 import java.util.ArrayList;
 import java.util.List;
 
@@ -126,28 +127,21 @@ public class AppletApplication extends HomeApplication {
 
     URL codeBase = applet.getCodeBase();
 
-    // Force offscreen in 3D view under Plugin 2 and Mac OS X
-    System.setProperty("com.eteks.sweethome3d.j3d.useOffScreen3DView", 
-        String.valueOf(OperatingSystem.isMacOSX()            
-            && applet.getAppletContext() != null
-            && applet.getAppletContext().getClass().getName().startsWith("sun.plugin2.applet.Plugin2Manager")));
-    
-    // If Sweet Home 3D applet is launched from outside of Java Web Start or basic service is unavailable
-    boolean serviceManagerAvailable = ServiceManager.getServiceNames() != null; 
-    if (serviceManagerAvailable) {
-      try { 
-        ServiceManager.lookup("javax.jnlp.BasicService");
-      } catch (UnavailableServiceException ex) {
-        serviceManagerAvailable = false;
-      }
+    try {
+      // Force offscreen in 3D view under Plugin 2 and Mac OS X
+      boolean plugin2 = applet.getAppletContext() != null
+                    && applet.getAppletContext().getClass().getName().startsWith("sun.plugin2.applet.Plugin2Manager");
+      System.setProperty("com.eteks.sweethome3d.j3d.useOffScreen3DView", 
+          String.valueOf(OperatingSystem.isMacOSX() && plugin2));
+      // Use DnD management without transfer handler under Plugin 2 + Mac OS X or OpenJDK + Linux 
+      System.setProperty("com.eteks.sweethome3d.dragAndDropWithoutTransferHandler", 
+          String.valueOf(OperatingSystem.isMacOSX() && plugin2 
+                        || (OperatingSystem.isLinux() && System.getProperty("java.runtime.name", "").startsWith("OpenJDK"))));
+    } catch (AccessControlException ex) {
+      // Unsigned applet
     }
-
-    if (!serviceManagerAvailable) {
-      // Create JNLP services required by Sweet Home 3D 
-      ServiceManager.setServiceManagerStub(
-          new StandaloneServiceManager(applet.getAppletContext(), codeBase));
-      // Caution: setting a new service manager stub won't replace the existing one,
-    }          
+    
+    checkJavaWebStartBasicService(applet, codeBase);          
  
     initLookAndFeel();
    
@@ -213,7 +207,7 @@ public class AppletApplication extends HomeApplication {
         }
       });
   }
-  
+
   /**
    * Deletes open homes and clears all the resources used by this application.
    * This method is called when an applet is destroyed.  
@@ -230,11 +224,16 @@ public class AppletApplication extends HomeApplication {
     }
     // Collect deleted objects (seems to be required under Mac OS X when the applet is being reloaded)
     System.gc();
-    // Stop managers threads
-    IconManager.getInstance().clear();
-    if (!"true".equalsIgnoreCase(System.getProperty("com.eteks.sweethome3d.no3D"))) { 
-      TextureManager.getInstance().clear();
-      ModelManager.getInstance().clear();
+    try {
+      if (!Boolean.getBoolean("com.eteks.sweethome3d.no3D")) { 
+        // Stop managers threads
+        TextureManager.getInstance().clear();
+        ModelManager.getInstance().clear();
+      }
+      IconManager.getInstance().clear();
+    } catch (AccessControlException ex) {
+      // If com.eteks.sweethome3d.no3D property can't be read, 
+      // security manager won't allow to access to Java 3D DLLs required by previous classes too
     }
     // Delete temporary files
     OperatingSystem.deleteTemporaryFiles();
@@ -479,13 +478,23 @@ public class AppletApplication extends HomeApplication {
     addToolBarAction(homeView, HomeView.ActionType.ZOOM_OUT, toolBar);
     addToolBarAction(homeView, HomeView.ActionType.ZOOM_IN, toolBar);
 
-    Action createPhotoAction = getToolBarAction(homeView, HomeView.ActionType.CREATE_PHOTO);
-    if (createPhotoAction != null) {
-      boolean enableCreatePhoto = getAppletBooleanParameter(this.applet, ENABLE_CREATE_PHOTO);
-      controller.getView().setEnabled(HomeView.ActionType.CREATE_PHOTO, enableCreatePhoto);
-      if (enableCreatePhoto) {
-        toolBar.addSeparator();
-        toolBar.add(createPhotoAction);
+    boolean no3D;
+    try {
+      no3D = Boolean.getBoolean("com.eteks.sweethome3d.no3D");
+    } catch (AccessControlException ex) {
+      // If com.eteks.sweethome3d.no3D property can't be read, 
+      // security manager won't allow to access to Java 3D DLLs required to manage 3D too
+      no3D = true;
+    }
+    if (!no3D) {
+      Action createPhotoAction = getToolBarAction(homeView, HomeView.ActionType.CREATE_PHOTO);
+      if (createPhotoAction != null) {
+        boolean enableCreatePhoto = getAppletBooleanParameter(this.applet, ENABLE_CREATE_PHOTO);
+        controller.getView().setEnabled(HomeView.ActionType.CREATE_PHOTO, enableCreatePhoto);
+        if (enableCreatePhoto) {
+          toolBar.addSeparator();
+          toolBar.add(createPhotoAction);
+        }
       }
     }
 
@@ -506,9 +515,9 @@ public class AppletApplication extends HomeApplication {
     controller.getView().setEnabled(HomeView.ActionType.EXPORT_TO_SVG, 
         getAppletBooleanParameter(this.applet, ENABLE_EXPORT_TO_SVG));
     controller.getView().setEnabled(HomeView.ActionType.EXPORT_TO_OBJ, 
-        getAppletBooleanParameter(this.applet, ENABLE_EXPORT_TO_OBJ));
+        getAppletBooleanParameter(this.applet, ENABLE_EXPORT_TO_OBJ) && !no3D);
     controller.getView().setEnabled(HomeView.ActionType.CREATE_VIDEO, 
-        getAppletBooleanParameter(this.applet, ENABLE_CREATE_VIDEO));
+        getAppletBooleanParameter(this.applet, ENABLE_CREATE_VIDEO) && !no3D);
     
     // Add a border
     homeView.setBorder(BorderFactory.createLineBorder(Color.LIGHT_GRAY));
@@ -627,9 +636,13 @@ public class AppletApplication extends HomeApplication {
   public String getVersion() {
     String applicationVersion = getUserPreferences().getLocalizedString(
         AppletApplication.class, "applicationVersion");
-    String versionInformation = System.getProperty("com.eteks.sweethome3d.deploymentInformation");
-    if (versionInformation != null) {
-      applicationVersion += " " + versionInformation;
+    try {
+      String versionInformation = System.getProperty("com.eteks.sweethome3d.deploymentInformation");
+      if (versionInformation != null) {
+        applicationVersion += " " + versionInformation;
+      }
+    } catch (AccessControlException ex) {
+      // Ignore com.eteks.sweethome3d.deploymentInformation property since it can't be read
     }
     return applicationVersion;
   }
@@ -659,21 +672,26 @@ public class AppletApplication extends HomeApplication {
    * to avoid default System exit in case of error during 3D rendering. 
    */
   private void addComponent3DRenderingErrorObserver() {
-    if (!"true".equalsIgnoreCase(System.getProperty("com.eteks.sweethome3d.no3D"))) { 
-      // Instead of adding a RenderingErrorListener directly to VirtualUniverse, 
-      // we add it through Component3DManager, because offscreen rendering needs to check 
-      // rendering errors with its own RenderingErrorListener
-      Component3DManager.getInstance().setRenderingErrorObserver(
-          new Component3DManager.RenderingErrorObserver() {
-            public void errorOccured(int errorCode, String errorMessage) {
-              System.err.print("Error in Java 3D : " + errorCode + " " + errorMessage);
-              EventQueue.invokeLater(new Runnable() {
-                  public void run() {
-                    show3DError();
-                  }
-                });
-            }
-          });
+    try {
+      if (!Boolean.getBoolean("com.eteks.sweethome3d.no3D")) {
+        // Instead of adding a RenderingErrorListener directly to VirtualUniverse, 
+        // we add it through Component3DManager, because offscreen rendering needs to check 
+        // rendering errors with its own RenderingErrorListener
+        Component3DManager.getInstance().setRenderingErrorObserver(
+            new Component3DManager.RenderingErrorObserver() {
+              public void errorOccured(int errorCode, String errorMessage) {
+                System.err.print("Error in Java 3D : " + errorCode + " " + errorMessage);
+                EventQueue.invokeLater(new Runnable() {
+                    public void run() {
+                      show3DError();
+                    }
+                  });
+              }
+            });
+      }
+    } catch (AccessControlException ex) {
+      // If com.eteks.sweethome3d.no3D property can't be read, 
+      // security manager won't allow to access to Java 3D DLLs required by Component3DManager class too
     }
   }
 
@@ -687,6 +705,31 @@ public class AppletApplication extends HomeApplication {
         message, title, JOptionPane.ERROR_MESSAGE);
   }
 
+  /**
+   * Checks whether Java Web Start basic service is available to be able to display document from the applet.
+   */
+  private void checkJavaWebStartBasicService(final JApplet applet, URL codeBase) {
+    boolean serviceManagerAvailable = ServiceManager.getServiceNames() != null; 
+    if (serviceManagerAvailable) {
+      try { 
+        ServiceManager.lookup("javax.jnlp.BasicService");
+      } catch (Exception ex) {
+        if ("javax.jnlp.UnavailableServiceException".equals(ex.getClass().getName())) {
+          serviceManagerAvailable = false;
+        } else {
+          throw new RuntimeException(ex);
+        }
+      }
+    }
+
+    if (!serviceManagerAvailable) {
+      // Create JNLP services required by Sweet Home 3D 
+      ServiceManager.setServiceManagerStub(
+          new StandaloneServiceManager(applet.getAppletContext(), codeBase));
+      // Caution: setting a new service manager stub won't replace the existing one
+    }
+  }
+
   /**
    * JNLP <code>ServiceManagerStub</code> implementation for applets 
    * run out of Java Web Start. This service manager supports <code>BasicService</code> only.
