@@ -26,6 +26,7 @@ import java.math.BigDecimal;
 import java.net.MalformedURLException;
 import java.net.URL;
 import java.net.URLClassLoader;
+import java.security.AccessControlException;
 import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.Collections;
@@ -190,6 +191,14 @@ public class DefaultFurnitureCatalog extends FurnitureCatalog {
      * The key for the color(s) of light sources in a light (optional).
      */
     LIGHT_SOURCE_DIAMETER("lightSourceDiameter"),
+    /**
+     * The key for the shape used to cut out upper levels when they intersect with a piece   
+     * like a staircase (optional). This shape should be defined with the syntax of 
+     * the d attribute of a <a href="http://www.w3.org/TR/SVG/paths.html">SVG path element</a>
+     * and should fit in a square spreading from (0, 0) to (1, 1) which will be scaled afterwards 
+     * to the real size of the piece. 
+     */
+    STAIRCASE_CUT_OUT_SHAPE("staircaseCutOutShape"),
     /**
      * The key for the elevation in centimeters of a piece of furniture (optional).
      */
@@ -231,7 +240,11 @@ public class DefaultFurnitureCatalog extends FurnitureCatalog {
     /**
      * The key for the VAT percentage of a piece of furniture (optional).
      */
-    VALUE_ADDED_TAX_PERCENTAGE("valueAddedTaxPercentage");
+    VALUE_ADDED_TAX_PERCENTAGE("valueAddedTaxPercentage"),
+    /**
+     * The key for the currency ISO 4217 code of the price of a piece of furniture (optional).
+     */
+    CURRENCY("currency");
     
     private String keyPrefix;
 
@@ -291,19 +304,7 @@ public class DefaultFurnitureCatalog extends FurnitureCatalog {
         new HashMap<FurnitureCategory, Map<CatalogPieceOfFurniture,Integer>>();
     List<String> identifiedFurniture = new ArrayList<String>();
     
-    // Try to load com.eteks.sweethome3d.io.DefaultFurnitureCatalog property file from classpath 
-    final String defaultFurnitureCatalogFamily = DefaultFurnitureCatalog.class.getName();
-    readFurnitureCatalog(defaultFurnitureCatalogFamily, 
-        preferences, furnitureHomonymsCounter, identifiedFurniture);
-    
-    // Try to load com.eteks.sweethome3d.io.ContributedFurnitureCatalog property file from classpath 
-    String classPackage = defaultFurnitureCatalogFamily.substring(0, defaultFurnitureCatalogFamily.lastIndexOf("."));
-    readFurnitureCatalog(classPackage + "." + CONTRIBUTED_FURNITURE_CATALOG_FAMILY, 
-        preferences, furnitureHomonymsCounter, identifiedFurniture);
-    
-    // Try to load com.eteks.sweethome3d.io.AdditionalFurnitureCatalog property file from classpath
-    readFurnitureCatalog(classPackage + "." + ADDITIONAL_FURNITURE_CATALOG_FAMILY, 
-        preferences, furnitureHomonymsCounter, identifiedFurniture);
+    readDefaultFurnitureCatalogs(preferences, furnitureHomonymsCounter, identifiedFurniture);
     
     if (furniturePluginFolders != null) {
       for (File furniturePluginFolder : furniturePluginFolders) {
@@ -326,7 +327,7 @@ public class DefaultFurnitureCatalog extends FurnitureCatalog {
       }
     }
   }
-  
+
   /**
    * Creates a default furniture catalog read only from resources in the given URLs.
    */
@@ -335,7 +336,8 @@ public class DefaultFurnitureCatalog extends FurnitureCatalog {
   }
   
   /**
-   * Creates a default furniture catalog read only from resources in the given URLs.
+   * Creates a default furniture catalog read only from resources in the given URLs 
+   * or in the classpath if the security manager doesn't allow to create class loaders.
    * Model and icon URLs will built from <code>furnitureResourcesUrlBase</code> if it isn't <code>null</code>.
    */
   public DefaultFurnitureCatalog(URL [] pluginFurnitureCatalogUrls,
@@ -344,16 +346,27 @@ public class DefaultFurnitureCatalog extends FurnitureCatalog {
         new HashMap<FurnitureCategory, Map<CatalogPieceOfFurniture,Integer>>();
     List<String> identifiedFurniture = new ArrayList<String>();
 
-    for (URL pluginFurnitureCatalogUrl : pluginFurnitureCatalogUrls) {
-      try {
-        readFurniture(ResourceBundle.getBundle(PLUGIN_FURNITURE_CATALOG_FAMILY, Locale.getDefault(), 
-                new URLClassLoader(new URL [] {pluginFurnitureCatalogUrl})), 
-            pluginFurnitureCatalogUrl, furnitureResourcesUrlBase, furnitureHomonymsCounter, identifiedFurniture);
-      } catch (MissingResourceException ex) {
-        // Ignore malformed furniture catalog
-      } catch (IllegalArgumentException ex) {
-        // Ignore malformed furniture catalog
+    try {
+      SecurityManager securityManager = System.getSecurityManager();
+      if (securityManager != null) {
+        securityManager.checkCreateClassLoader();
       }
+
+      for (URL pluginFurnitureCatalogUrl : pluginFurnitureCatalogUrls) {
+        try {        
+          ResourceBundle resource = ResourceBundle.getBundle(PLUGIN_FURNITURE_CATALOG_FAMILY, Locale.getDefault(), 
+              new URLClassLoader(new URL [] {pluginFurnitureCatalogUrl}));
+          readFurniture(resource, pluginFurnitureCatalogUrl, furnitureResourcesUrlBase, furnitureHomonymsCounter, identifiedFurniture);
+        } catch (MissingResourceException ex) {
+          // Ignore malformed furniture catalog
+        } catch (IllegalArgumentException ex) {
+          // Ignore malformed furniture catalog
+        }
+      }
+    } catch (AccessControlException ex) {
+      // Use only furniture accessible through classpath
+      ResourceBundle resource = ResourceBundle.getBundle(PLUGIN_FURNITURE_CATALOG_FAMILY, Locale.getDefault());
+      readFurniture(resource, null, furnitureResourcesUrlBase, furnitureHomonymsCounter, identifiedFurniture);
     }
   }
 
@@ -395,6 +408,27 @@ public class DefaultFurnitureCatalog extends FurnitureCatalog {
     }
   }
   
+  /**
+   * Reads the default furniture described in properties files accessible through classpath.
+   */
+  private void readDefaultFurnitureCatalogs(UserPreferences preferences,
+                                            Map<FurnitureCategory, Map<CatalogPieceOfFurniture, Integer>> furnitureHomonymsCounter,
+                                            List<String> identifiedFurniture) {
+    // Try to load com.eteks.sweethome3d.io.DefaultFurnitureCatalog property file from classpath 
+    String defaultFurnitureCatalogFamily = DefaultFurnitureCatalog.class.getName();
+    readFurnitureCatalog(defaultFurnitureCatalogFamily, 
+        preferences, furnitureHomonymsCounter, identifiedFurniture);
+    
+    // Try to load com.eteks.sweethome3d.io.ContributedFurnitureCatalog property file from classpath 
+    String classPackage = defaultFurnitureCatalogFamily.substring(0, defaultFurnitureCatalogFamily.lastIndexOf("."));
+    readFurnitureCatalog(classPackage + "." + CONTRIBUTED_FURNITURE_CATALOG_FAMILY, 
+        preferences, furnitureHomonymsCounter, identifiedFurniture);
+    
+    // Try to load com.eteks.sweethome3d.io.AdditionalFurnitureCatalog property file from classpath
+    readFurnitureCatalog(classPackage + "." + ADDITIONAL_FURNITURE_CATALOG_FAMILY, 
+        preferences, furnitureHomonymsCounter, identifiedFurniture);
+  }
+  
   /**
    * Reads furniture of a given catalog family from resources.
    */
@@ -496,6 +530,7 @@ public class DefaultFurnitureCatalog extends FurnitureCatalog {
     float height = Float.parseFloat(resource.getString(PropertyKey.HEIGHT.getKey(index)));
     boolean movable = Boolean.parseBoolean(resource.getString(PropertyKey.MOVABLE.getKey(index)));
     boolean doorOrWindow = Boolean.parseBoolean(resource.getString(PropertyKey.DOOR_OR_WINDOW.getKey(index)));
+    String staircaseCutOutShape = getOptionalString(resource, PropertyKey.STAIRCASE_CUT_OUT_SHAPE.getKey(index), null);     
     float elevation = getOptionalFloat(resource, PropertyKey.ELEVATION.getKey(index), 0);
     float [][] modelRotation = getModelRotation(resource, PropertyKey.MODEL_ROTATION.getKey(index));
     // By default creator is eTeks
@@ -516,6 +551,7 @@ public class DefaultFurnitureCatalog extends FurnitureCatalog {
     } catch (MissingResourceException ex) {
       // By default price is null
     }
+    String currency = getOptionalString(resource, PropertyKey.CURRENCY.getKey(index), null);
 
     if (doorOrWindow) {
       float wallThicknessPercentage = getOptionalFloat(
@@ -526,17 +562,17 @@ public class DefaultFurnitureCatalog extends FurnitureCatalog {
       return new CatalogDoorOrWindow(id, name, description, icon, planIcon, model,
           width, depth, height, elevation, movable, 
           wallThicknessPercentage, wallDistancePercentage, sashes, modelRotation, creator, 
-          resizable, deformable, texturable, price, valueAddedTaxPercentage);
+          resizable, deformable, texturable, price, valueAddedTaxPercentage, currency);
     } else {
       LightSource [] lightSources = getLightSources(resource, index, width, depth, height);
       if (lightSources != null) {
         return new CatalogLight(id, name, description, icon, planIcon, model,
-            width, depth, height, elevation, movable, lightSources, modelRotation, creator, 
-            resizable, deformable, texturable, price, valueAddedTaxPercentage);
+            width, depth, height, elevation, movable, lightSources, staircaseCutOutShape, modelRotation, creator, 
+            resizable, deformable, texturable, price, valueAddedTaxPercentage, currency);
       } else {
         return new CatalogPieceOfFurniture(id, name, description, icon, planIcon, model,
-            width, depth, height, elevation, movable, modelRotation, creator, 
-            resizable, deformable, texturable, price, valueAddedTaxPercentage);
+            width, depth, height, elevation, movable, staircaseCutOutShape, modelRotation, creator, 
+            resizable, deformable, texturable, price, valueAddedTaxPercentage, currency);
       }
     }
   }
@@ -584,23 +620,24 @@ public class DefaultFurnitureCatalog extends FurnitureCatalog {
             doorOrWindow.getWallDistance(), doorOrWindow.getSashes(), 
             doorOrWindow.getModelRotation(), doorOrWindow.getCreator(),
             doorOrWindow.isResizable(), doorOrWindow.isDeformable(), doorOrWindow.isTexturable(), 
-            doorOrWindow.getPrice(), doorOrWindow.getValueAddedTaxPercentage());
+            doorOrWindow.getPrice(), doorOrWindow.getValueAddedTaxPercentage(), doorOrWindow.getCurrency());
       } else if (piece instanceof CatalogLight) {
         CatalogLight light = (CatalogLight)piece;
         piece = new CatalogLight(light.getId(), suffixedName,
             light.getDescription(), light.getIcon(), light.getPlanIcon(), light.getModel(),
             light.getWidth(), light.getDepth(), light.getHeight(), light.getElevation(), 
-            light.isMovable(), light.getLightSources(), 
+            light.isMovable(), light.getLightSources(), light.getStaircaseCutOutShape(), 
             light.getModelRotation(), light.getCreator(),
             light.isResizable(), light.isDeformable(), light.isTexturable(),
-            light.getPrice(), light.getValueAddedTaxPercentage());
+            light.getPrice(), light.getValueAddedTaxPercentage(), light.getCurrency());
       } else {
         piece = new CatalogPieceOfFurniture(piece.getId(), suffixedName,
             piece.getDescription(), piece.getIcon(), piece.getPlanIcon(), piece.getModel(),
-            piece.getWidth(), piece.getDepth(), piece.getHeight(), piece.getElevation(), 
-            piece.isMovable(), piece.getModelRotation(), piece.getCreator(),
+            piece.getWidth(), piece.getDepth(), piece.getHeight(), 
+            piece.getElevation(), piece.isMovable(), piece.getStaircaseCutOutShape(), 
+            piece.getModelRotation(), piece.getCreator(),
             piece.isResizable(), piece.isDeformable(), piece.isTexturable(),
-            piece.getPrice(), piece.getValueAddedTaxPercentage());
+            piece.getPrice(), piece.getValueAddedTaxPercentage(), piece.getCurrency());
       }
       add(pieceCategory, piece, furnitureHomonymsCounter);
     }
@@ -633,14 +670,15 @@ public class DefaultFurnitureCatalog extends FurnitureCatalog {
       // Try first to interpret contentFile as an absolute URL 
       // or an URL relative to resourceUrlBase if it's not null
       URL url;
-      if (resourceUrlBase != null) {
-        if (contentFile.indexOf('!') < 0 || contentFile.startsWith("jar:")) {
-          url = new URL(resourceUrlBase, contentFile);
-        } else {
-          url = new URL("jar:" + new URL(resourceUrlBase, contentFile));
-        }
-      } else {
+      if (resourceUrlBase == null) {
         url = new URL(contentFile);
+      } else {
+        url = contentFile.startsWith("?") 
+            ? new URL(resourceUrlBase + contentFile)
+            : new URL(resourceUrlBase, contentFile);
+        if (contentFile.indexOf('!') >= 0 && !contentFile.startsWith("jar:")) {
+          url = new URL("jar:" + url);
+        }
       }
       return new URLContent(url);
     } catch (MalformedURLException ex) {
