@@ -25,6 +25,7 @@ import java.io.IOException;
 import java.net.MalformedURLException;
 import java.net.URL;
 import java.net.URLClassLoader;
+import java.security.AccessControlException;
 import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.Collections;
@@ -145,15 +146,7 @@ public class DefaultTexturesCatalog extends TexturesCatalog {
         new HashMap<TexturesCategory, Map<CatalogTexture,Integer>>();
     List<String> identifiedTextures = new ArrayList<String>();
     
-    // Try to load com.eteks.sweethome3d.io.DefaultTexturesCatalog property file from classpath 
-    final String defaultTexturesCatalogFamily = DefaultTexturesCatalog.class.getName();
-    readTexturesCatalog(defaultTexturesCatalogFamily, 
-        preferences, textureHomonymsCounter, identifiedTextures);
-
-    // Try to load com.eteks.sweethome3d.io.AdditionalTexturesCatalog property file from classpath 
-    String classPackage = defaultTexturesCatalogFamily.substring(0, defaultTexturesCatalogFamily.lastIndexOf("."));
-    readTexturesCatalog(classPackage + "." + ADDITIONAL_TEXTURES_CATALOG_FAMILY, 
-        preferences, textureHomonymsCounter, identifiedTextures);
+    readDefaultTexturesCatalogs(preferences, textureHomonymsCounter, identifiedTextures);
 
     if (texturesPluginFolders != null) {
       for (File texturesPluginFolder : texturesPluginFolders) {
@@ -176,7 +169,7 @@ public class DefaultTexturesCatalog extends TexturesCatalog {
       }
     }
   }
-  
+
   /**
    * Creates a default textures catalog read only from resources in the given URLs.
    */
@@ -194,16 +187,27 @@ public class DefaultTexturesCatalog extends TexturesCatalog {
         new HashMap<TexturesCategory, Map<CatalogTexture,Integer>>();
     List<String> identifiedTextures = new ArrayList<String>();
 
-    for (URL pluginTexturesCatalogUrl : pluginTexturesCatalogUrls) {
-      try {
-        readTextures(ResourceBundle.getBundle(PLUGIN_TEXTURES_CATALOG_FAMILY, Locale.getDefault(),
-                new URLClassLoader(new URL [] {pluginTexturesCatalogUrl})),
-            pluginTexturesCatalogUrl, texturesResourcesUrlBase, textureHomonymsCounter, identifiedTextures);
-      } catch (MissingResourceException ex) {
-        // Ignore malformed textures catalog
-      } catch (IllegalArgumentException ex) {
-        // Ignore malformed textures catalog
+    try {
+      SecurityManager securityManager = System.getSecurityManager();
+      if (securityManager != null) {
+        securityManager.checkCreateClassLoader();
       }
+
+      for (URL pluginTexturesCatalogUrl : pluginTexturesCatalogUrls) {
+        try {
+          ResourceBundle resource = ResourceBundle.getBundle(PLUGIN_TEXTURES_CATALOG_FAMILY, Locale.getDefault(),
+                  new URLClassLoader(new URL [] {pluginTexturesCatalogUrl}));
+          readTextures(resource, pluginTexturesCatalogUrl, texturesResourcesUrlBase, textureHomonymsCounter, identifiedTextures);
+        } catch (MissingResourceException ex) {
+          // Ignore malformed textures catalog
+        } catch (IllegalArgumentException ex) {
+          // Ignore malformed textures catalog
+        }
+      }
+    } catch (AccessControlException ex) {
+      // Use only textures accessible through classpath
+      ResourceBundle resource = ResourceBundle.getBundle(PLUGIN_TEXTURES_CATALOG_FAMILY, Locale.getDefault());
+      readTextures(resource, null, texturesResourcesUrlBase, textureHomonymsCounter, identifiedTextures);
     }
   }
 
@@ -245,6 +249,23 @@ public class DefaultTexturesCatalog extends TexturesCatalog {
     }
   }
   
+  /**
+   * Reads the default textures described in properties files accessible through classpath.
+   */
+  private void readDefaultTexturesCatalogs(final UserPreferences preferences,
+                                           Map<TexturesCategory, Map<CatalogTexture, Integer>> textureHomonymsCounter,
+                                           List<String> identifiedTextures) {
+    // Try to load com.eteks.sweethome3d.io.DefaultTexturesCatalog property file from classpath 
+    final String defaultTexturesCatalogFamily = DefaultTexturesCatalog.class.getName();
+    readTexturesCatalog(defaultTexturesCatalogFamily, 
+        preferences, textureHomonymsCounter, identifiedTextures);
+
+    // Try to load com.eteks.sweethome3d.io.AdditionalTexturesCatalog property file from classpath 
+    String classPackage = defaultTexturesCatalogFamily.substring(0, defaultTexturesCatalogFamily.lastIndexOf("."));
+    readTexturesCatalog(classPackage + "." + ADDITIONAL_TEXTURES_CATALOG_FAMILY, 
+        preferences, textureHomonymsCounter, identifiedTextures);
+  }
+  
   /**
    * Reads textures of a given catalog family from resources.
    */
@@ -372,10 +393,12 @@ public class DefaultTexturesCatalog extends TexturesCatalog {
       // Try first to interpret contentFile as an absolute URL 
       // or an URL relative to resourceUrlBase if it's not null
       URL url;
-      if (resourceUrlBase != null) {
-        url = new URL(resourceUrlBase, contentFile);
-      } else {
+      if (resourceUrlBase == null) {
         url = new URL(contentFile);
+      } else {
+        url = contentFile.startsWith("?") 
+            ? new URL(resourceUrlBase + contentFile)
+            : new URL(resourceUrlBase, contentFile);
       }
       return new URLContent(url);
     } catch (MalformedURLException ex) {
