@@ -40,6 +40,7 @@ import java.util.HashSet;
 import java.util.Iterator;
 import java.util.LinkedHashSet;
 import java.util.List;
+import java.util.Locale;
 import java.util.Map;
 import java.util.Map.Entry;
 import java.util.Properties;
@@ -88,7 +89,10 @@ public class FileUserPreferences extends UserPreferences {
   private static final String WALL_PATTERN                          = "wallPattern";
   private static final String NEW_WALL_HEIGHT                       = "newHomeWallHeight";
   private static final String NEW_WALL_THICKNESS                    = "newWallThickness";
+  private static final String NEW_FLOOR_THICKNESS                   = "newFloorThickness";
   private static final String AUTO_SAVE_DELAY_FOR_RECOVERY          = "autoSaveDelayForRecovery";
+  private static final String AUTO_COMPLETION_PROPERTY              = "autoCompletionProperty#";
+  private static final String AUTO_COMPLETION_STRINGS               = "autoCompletionStrings#";
   private static final String RECENT_HOMES                          = "recentHomes#";
   private static final String IGNORED_ACTION_TIP                    = "ignoredActionTip#";  
 
@@ -104,6 +108,7 @@ public class FileUserPreferences extends UserPreferences {
   private static final String FURNITURE_ELEVATION                   = "furnitureElevation#";
   private static final String FURNITURE_COLOR                       = "furnitureColor#";
   private static final String FURNITURE_MODEL_ROTATION              = "furnitureModelRotation#";
+  private static final String FURNITURE_STAIRCASE_CUT_OUT_SHAPE           = "furnitureStaircaseCutOutShape#"; 
   private static final String FURNITURE_BACK_FACE_SHOWN             = "furnitureBackFaceShown#";
   private static final String FURNITURE_ICON_YAW                    = "furnitureIconYaw#";
   private static final String FURNITURE_PROPORTIONAL                = "furnitureProportional#";
@@ -125,7 +130,6 @@ public class FileUserPreferences extends UserPreferences {
   
   private final Map<String, Boolean> ignoredActionTips = new HashMap<String, Boolean>();
   private List<ClassLoader>          resourceClassLoaders;
-  private String []                  supportedLanguages;
   private final File                 preferencesFolder;
   private final File []              applicationFolders;
   private Preferences                preferences;
@@ -175,7 +179,12 @@ public class FileUserPreferences extends UserPreferences {
       preferences = portablePreferences;
     }
     
-    setLanguage(preferences.get(LANGUAGE, getLanguage()));    
+    String language = preferences.get(LANGUAGE, getLanguage());
+    // Check language is still supported
+    if (!Arrays.asList(getSupportedLanguages()).contains(language)) {
+      language = Locale.ENGLISH.getLanguage();  
+    }
+    setLanguage(language);    
     
     // Fill default furniture catalog 
     setFurnitureCatalog(new DefaultFurnitureCatalog(this, getFurnitureLibrariesPluginFolders()));
@@ -187,9 +196,7 @@ public class FileUserPreferences extends UserPreferences {
     // Read additional textures
     readTexturesCatalog(preferences);
 
-    DefaultUserPreferences defaultPreferences = new DefaultUserPreferences();
-    // Share same language settings 
-    defaultPreferences.setLanguage(getLanguage());
+    DefaultUserPreferences defaultPreferences = new DefaultUserPreferences(false, this);
     
     // Fill default patterns catalog 
     PatternsCatalog patternsCatalog = defaultPreferences.getPatternsCatalog();
@@ -222,8 +229,10 @@ public class FileUserPreferences extends UserPreferences {
         defaultPreferences.getNewWallThickness()));
     setNewWallHeight(preferences.getFloat(NEW_WALL_HEIGHT,
         defaultPreferences.getNewWallHeight()));    
+    setNewFloorThickness(preferences.getFloat(NEW_FLOOR_THICKNESS, 
+        defaultPreferences.getNewFloorThickness()));
     setAutoSaveDelayForRecovery(preferences.getInt(AUTO_SAVE_DELAY_FOR_RECOVERY,
-        defaultPreferences.getAutoSaveDelayForRecovery()));    
+        defaultPreferences.getAutoSaveDelayForRecovery()));
     setCurrency(defaultPreferences.getCurrency());    
     // Read recent homes list
     List<String> recentHomes = new ArrayList<String>();
@@ -243,11 +252,26 @@ public class FileUserPreferences extends UserPreferences {
         this.ignoredActionTips.put(ignoredActionTip, true);
       }
     }
+    // Get default auto completion strings
+    for (String property : defaultPreferences.getAutoCompletedProperties()) {
+      setAutoCompletionStrings(property, defaultPreferences.getAutoCompletionStrings(property));
+    }
+    // Read auto completion strings list
+    for (int i = 1; ; i++) {
+      String autoCompletionProperty = preferences.get(AUTO_COMPLETION_PROPERTY + i, null);
+      String autoCompletionStrings = preferences.get(AUTO_COMPLETION_STRINGS + i, null);
+      if (autoCompletionProperty != null && autoCompletionStrings != null) {
+        setAutoCompletionStrings(autoCompletionProperty, Arrays.asList(autoCompletionStrings.split(",")));
+      } else {
+        break;
+      }
+    }
     
     addPropertyChangeListener(Property.LANGUAGE, new PropertyChangeListener() {
-        public void propertyChange(PropertyChangeEvent evt) {
+        public void propertyChange(PropertyChangeEvent ev) {
           updateFurnitureDefaultCatalog();
           updateTexturesDefaultCatalog();
+          updateAutoCompletionStrings();
         }
       });
     
@@ -263,7 +287,7 @@ public class FileUserPreferences extends UserPreferences {
    */
   private void updateSupportedLanguages() {
     List<ClassLoader> resourceClassLoaders = new ArrayList<ClassLoader>();
-    String [] defaultSupportedLanguages = super.getSupportedLanguages();
+    String [] defaultSupportedLanguages = getDefaultSupportedLanguages();
     Set<String> supportedLanguages = new TreeSet<String>(Arrays.asList(defaultSupportedLanguages));
    
     File [] languageLibrariesPluginFolders = getLanguageLibrariesPluginFolders();
@@ -300,9 +324,7 @@ public class FileUserPreferences extends UserPreferences {
     resourceClassLoaders.addAll(super.getResourceClassLoaders());
     this.resourceClassLoaders = Collections.unmodifiableList(resourceClassLoaders);
     if (defaultSupportedLanguages.length < supportedLanguages.size()) {
-      this.supportedLanguages = supportedLanguages.toArray(new String [supportedLanguages.size()]);
-    } else {
-      this.supportedLanguages = defaultSupportedLanguages;
+      setSupportedLanguages(supportedLanguages.toArray(new String [supportedLanguages.size()]));
     }
   }
 
@@ -340,14 +362,6 @@ public class FileUserPreferences extends UserPreferences {
     }
   }
   
-  /**
-   * Returns the default languages supported in Sweet Home 3D and languages in plugin libraries.
-   */
-  @Override
-  public String [] getSupportedLanguages() {    
-    return this.supportedLanguages;
-  }
-  
   /**
    * Returns the default class loader of user preferences and the class loaders that
    * give access to resources in language libraries plugin folder. 
@@ -413,6 +427,18 @@ public class FileUserPreferences extends UserPreferences {
     }
   }
 
+  /**
+   * Adds to auto completion strings the default strings of the new chosen language.
+   */
+  private void updateAutoCompletionStrings() {
+    DefaultUserPreferences defaultPreferences = new DefaultUserPreferences(false, this);
+    for (String property : defaultPreferences.getAutoCompletedProperties()) {
+      for (String autoCompletionString : defaultPreferences.getAutoCompletionStrings(property)) {
+        addAutoCompletionString(property, autoCompletionString);
+      }
+    }
+  }
+
   /**
    * Read furniture catalog from preferences.
    */
@@ -436,6 +462,7 @@ public class FileUserPreferences extends UserPreferences {
       Integer color = colorString != null 
           ? Integer.valueOf(colorString) : null; 
       float [][] modelRotation = getModelRotation(preferences, FURNITURE_MODEL_ROTATION + i);
+      String staircaseCutOutShape = preferences.get(FURNITURE_STAIRCASE_CUT_OUT_SHAPE + i, null);
       boolean backFaceShown = preferences.getBoolean(FURNITURE_BACK_FACE_SHOWN + i, false);
       float iconYaw = preferences.getFloat(FURNITURE_ICON_YAW + i, 0);
       boolean proportional = preferences.getBoolean(FURNITURE_PROPORTIONAL + i, true);
@@ -449,7 +476,7 @@ public class FileUserPreferences extends UserPreferences {
       } else {
         piece = new CatalogPieceOfFurniture(name, icon, model,
             width, depth, height, elevation, movable, 
-            color, modelRotation, backFaceShown, iconYaw, proportional);
+            staircaseCutOutShape, color, modelRotation, backFaceShown, iconYaw, proportional);
       }
       try {        
         getFurnitureCatalog().add(pieceCategory, piece);
@@ -572,6 +599,7 @@ public class FileUserPreferences extends UserPreferences {
     preferences.put(WALL_PATTERN, getWallPattern().getName());
     preferences.putFloat(NEW_WALL_THICKNESS, getNewWallThickness());   
     preferences.putFloat(NEW_WALL_HEIGHT, getNewWallHeight());
+    preferences.putFloat(NEW_FLOOR_THICKNESS, getNewFloorThickness());   
     preferences.putInt(AUTO_SAVE_DELAY_FOR_RECOVERY, getAutoSaveDelayForRecovery());
     // Write recent homes list
     int i = 1;
@@ -595,6 +623,28 @@ public class FileUserPreferences extends UserPreferences {
     for ( ; i <= this.ignoredActionTips.size(); i++) {
       preferences.remove(IGNORED_ACTION_TIP + i);
     }
+    // Write auto completion strings lists
+    i = 1;
+    for (String property : getAutoCompletedProperties()) {
+      StringBuilder autoCompletionStrings = new StringBuilder();
+      Iterator<String> it = getAutoCompletionStrings(property).iterator();
+      for (int j = 0; j < 1000 && it.hasNext(); j++) {
+        String autoCompletionString = it.next();
+        // As strings are comma separated, accept only the ones without a comma 
+        if (autoCompletionString.indexOf(',') < 0) {
+          if (autoCompletionStrings.length() > 0) {
+            autoCompletionStrings.append(",");
+          } 
+          autoCompletionStrings.append(autoCompletionString);
+        }
+      }
+      preferences.put(AUTO_COMPLETION_PROPERTY + i, property);
+      preferences.put(AUTO_COMPLETION_STRINGS + i++, autoCompletionStrings.toString());
+    }
+    for ( ; preferences.get(AUTO_COMPLETION_PROPERTY + i, null) != null; i++) {
+      preferences.remove(AUTO_COMPLETION_PROPERTY + i);
+      preferences.remove(AUTO_COMPLETION_STRINGS + i);
+    }
     
     try {
       // Write preferences 
@@ -632,9 +682,12 @@ public class FileUserPreferences extends UserPreferences {
           }
           float [][] modelRotation = piece.getModelRotation();
           preferences.put(FURNITURE_MODEL_ROTATION + i, 
-              modelRotation[0][0] + " " + modelRotation[0][1] + " " + modelRotation[0][2] + " "
-              + modelRotation[1][0] + " " + modelRotation[1][1] + " " + modelRotation[1][2] + " "
-              + modelRotation[2][0] + " " + modelRotation[2][1] + " " + modelRotation[2][2]);
+              floatToString(modelRotation[0][0]) + " " + floatToString(modelRotation[0][1]) + " " + floatToString(modelRotation[0][2]) + " "
+              + floatToString(modelRotation[1][0]) + " " + floatToString(modelRotation[1][1]) + " " + floatToString(modelRotation[1][2]) + " "
+              + floatToString(modelRotation[2][0]) + " " + floatToString(modelRotation[2][1]) + " " + floatToString(modelRotation[2][2]));
+          if (piece.getStaircaseCutOutShape() != null) {
+            preferences.put(FURNITURE_STAIRCASE_CUT_OUT_SHAPE + i, piece.getStaircaseCutOutShape());
+          }
           preferences.putBoolean(FURNITURE_BACK_FACE_SHOWN + i, piece.isBackFaceShown());
           preferences.putFloat(FURNITURE_ICON_YAW + i, piece.getIconYaw());
           preferences.putBoolean(FURNITURE_PROPORTIONAL + i, piece.isProportional());
@@ -656,6 +709,7 @@ public class FileUserPreferences extends UserPreferences {
       preferences.remove(FURNITURE_ELEVATION + i);
       preferences.remove(FURNITURE_COLOR + i);
       preferences.remove(FURNITURE_MODEL_ROTATION + i);
+      preferences.remove(FURNITURE_STAIRCASE_CUT_OUT_SHAPE + i);
       preferences.remove(FURNITURE_BACK_FACE_SHOWN + i);
       preferences.remove(FURNITURE_ICON_YAW + i);
       preferences.remove(FURNITURE_PROPORTIONAL + i);
@@ -663,6 +717,21 @@ public class FileUserPreferences extends UserPreferences {
     deleteObsoleteContent(furnitureContentURLs, FURNITURE_CONTENT_PREFIX);
   }
 
+  /**
+   * Returns the string value of the given float, except for -1.0, 1.0 or 0.0 where -1, 1 and 0 is returned.
+   */
+  private String floatToString(float f) {
+    if (Math.abs(f) < 1E-6) {
+      return "0";
+    } else if (Math.abs(f - 1f) < 1E-6) {
+      return "1";
+    } else if (Math.abs(f + 1f) < 1E-6) {
+      return "-1";
+    } else {
+      return String.valueOf(f);
+    }
+  }
+    
   /**
    * Writes textures catalog in <code>preferences</code>.
    */
@@ -973,11 +1042,6 @@ public class FileUserPreferences extends UserPreferences {
       File languageLibraryFile = new File(languageLibraryName);
       copyToLibraryFolder(languageLibraryFile, languageLibrariesPluginFolders [0]);
       updateSupportedLanguages();
-      // Switch automatically to the first language contained in library
-      Set<String> languages = getLanguages(languageLibraryFile);
-      if (!languages.isEmpty()) {
-        setLanguage(languages.iterator().next());
-      }
     } catch (IOException ex) {
       throw new RecorderException(
           "Can't write " + languageLibraryName +  " in language libraries plugin folder", ex);
