@@ -101,6 +101,7 @@ import com.eteks.sweethome3d.model.HomeFurnitureGroup;
 import com.eteks.sweethome3d.model.HomeLight;
 import com.eteks.sweethome3d.model.HomePieceOfFurniture;
 import com.eteks.sweethome3d.model.HomeTexture;
+import com.eteks.sweethome3d.model.Level;
 import com.eteks.sweethome3d.model.LightSource;
 import com.eteks.sweethome3d.model.ObserverCamera;
 import com.eteks.sweethome3d.model.Room;
@@ -146,7 +147,7 @@ public class PhotoRenderer {
         new Object3DBranchFactory() {
           public Object createObject3D(Home home, Selectable item, boolean waitForLoading) {
             if (item instanceof Room) {
-              // Never display ceiling whit top camera
+              // Never display ceiling with top camera
               return new Room3D((Room)item, home, home.getCamera() == home.getTopCamera(), true, waitForLoading);
             } else {
               return super.createObject3D(home, item, waitForLoading);
@@ -185,11 +186,8 @@ public class PhotoRenderer {
     for (Room room : home.getRooms()) {
       exportNode((Node)object3DFactory.createObject3D(home, room, true), true, false, silk);
     } 
-    // Create a dummy home to export a ground 3D not cut by rooms and large enough to join the sky at the horizon  
-    Home groundHome = new Home();
-    groundHome.getEnvironment().setGroundColor(home.getEnvironment().getGroundColor());
-    groundHome.getEnvironment().setGroundTexture(home.getEnvironment().getGroundTexture());
-    Ground3D ground = new Ground3D(groundHome, -1E7f / 2, -1E7f / 2, 1E7f, 1E7f, true);
+    // Create a 3D ground large enough to join the sky at the horizon  
+    Ground3D ground = new Ground3D(home, -1E7f / 2, -1E7f / 2, 1E7f, 1E7f, true);
     Transform3D translation = new Transform3D();
     translation.setTranslation(new Vector3f(0, -0.1f, 0));
     TransformGroup groundTransformGroup = new TransformGroup(translation);
@@ -227,29 +225,41 @@ public class PhotoRenderer {
     if (ceillingLightColor > 0) {
       // Add lights at the top of each room 
       for (Room room : home.getRooms()) {
-        if (room.isCeilingVisible()) {
+        Level roomLevel = room.getLevel();
+        if (room.isCeilingVisible() 
+            && (roomLevel == null || roomLevel.isVisible())) {
           float xCenter = room.getXCenter();
           float yCenter = room.getYCenter();
           
           double smallestDistance = Float.POSITIVE_INFINITY;
-          float roomHeight = home.getWallHeight();
-          
-          // Search the height of the wall closest to the point xCenter, yCenter
-          for (Wall wall : home.getWalls()) {
-            Float wallHeightAtStart = wall.getHeight();
-            float [][] points = wall.getPoints();
-            for (int i = 0; i < points.length; i++) {
-              double distanceToWallPoint = Point2D.distanceSq(points [i][0], points [i][1], xCenter, yCenter);
-              if (distanceToWallPoint < smallestDistance) {
-                smallestDistance = distanceToWallPoint; 
-                if (i == 0 || i == 3) { // Wall start
-                  roomHeight = wallHeightAtStart != null 
-                      ? wallHeightAtStart 
-                      : home.getWallHeight();
-                } else { // Wall end
-                  roomHeight = wall.isTrapezoidal() 
-                      ? wall.getHeightAtEnd() 
-                      : (wallHeightAtStart != null ? wallHeightAtStart : home.getWallHeight());
+          float roomElevation = roomLevel != null
+              ? roomLevel.getElevation()
+              : 0;
+          float roomHeight = roomElevation + 
+              (roomLevel == null ? home.getWallHeight() : roomLevel.getHeight());
+          List<Level> levels = home.getLevels();
+          if (roomLevel == null || levels.indexOf(roomLevel) == levels.size() - 1) {
+            // Search the height of the wall closest to the point xCenter, yCenter
+            for (Wall wall : home.getWalls()) {
+              if (wall.isAtLevel(roomLevel)) {
+                float wallElevation = wall.getLevel() == null ? 0 : wall.getLevel().getElevation();
+                Float wallHeightAtStart = wall.getHeight();
+                float [][] points = wall.getPoints();
+                for (int i = 0; i < points.length; i++) {
+                  double distanceToWallPoint = Point2D.distanceSq(points [i][0], points [i][1], xCenter, yCenter);
+                  if (distanceToWallPoint < smallestDistance) {
+                    smallestDistance = distanceToWallPoint; 
+                    if (i == 0 || i == points.length - 1) { // Wall start
+                      roomHeight = wallHeightAtStart != null 
+                          ? wallHeightAtStart 
+                          : home.getWallHeight();
+                    } else { // Wall end
+                      roomHeight = wall.isTrapezoidal() 
+                          ? wall.getHeightAtEnd() 
+                          : (wallHeightAtStart != null ? wallHeightAtStart : home.getWallHeight());
+                    }
+                    roomHeight += wallElevation;
+                  }
                 }
               }
             }
@@ -271,8 +281,11 @@ public class PhotoRenderer {
     // Add visible and turned on lights
     for (HomeLight light : getLights(home.getFurniture())) {
       float lightPower = light.getPower();
+      Level level = light.getLevel();
       if (light.isVisible()
-          && lightPower > 0f) {
+          && lightPower > 0f
+          && (level == null
+              || level.isVisible())) {
         float angle = light.getAngle();
         float cos = (float)Math.cos(angle);
         float sin = (float)Math.sin(angle);
@@ -288,9 +301,10 @@ public class PhotoRenderer {
               power * (lightColor & 0xFF) * (this.homeLightColor & 0xFF));
           float xLightSourceInLight = -light.getWidth() / 2 + (lightSource.getX() * light.getWidth());
           float yLightSourceInLight = light.getDepth() / 2 - (lightSource.getY() * light.getDepth());
+          float lightElevation = light.getGroundElevation();
           this.sunflow.parameter("center",
               new Point3(light.getX() + xLightSourceInLight * cos - yLightSourceInLight * sin,
-                  light.getElevation() + (lightSource.getZ() * light.getHeight()),
+                  lightElevation + (lightSource.getZ() * light.getHeight()),
                   light.getY() + xLightSourceInLight * sin + yLightSourceInLight * cos));                    
           this.sunflow.parameter("radius", lightRadius);
           this.sunflow.parameter("samples", 4);
@@ -490,7 +504,7 @@ public class PhotoRenderer {
   }
 
   /**
-   * Exports the given Java 3D <code>node</code> and its children to Sunflow API.  
+   * Exports the given Java 3D <code>node</code> and its children with SunFlow API.  
    */
   private void exportNode(Node node, boolean ignoreTransparency, 
                           boolean ignoreConstantShader, boolean silk) throws IOException {
@@ -498,7 +512,7 @@ public class PhotoRenderer {
   }
 
   /**
-   * Exports all the 3D shapes children of <code>node</code> at OBJ format.
+   * Exports all the 3D shapes children of <code>node</code> with SunFlow API.
    */ 
   private void exportNode(Node node, 
                           boolean ignoreTransparency,
@@ -565,7 +579,7 @@ public class PhotoRenderer {
   }
   
   /**
-   * Returns the names of the exported 3D geometries in Sunflow API.
+   * Returns the names of the exported 3D geometries with SunFlow API.
    */
   private String [] exportNodeGeometry(Geometry geometry, 
                                        Transform3D parentTransformations, 
@@ -574,7 +588,7 @@ public class PhotoRenderer {
     if (geometry instanceof GeometryArray) {
       GeometryArray geometryArray = (GeometryArray)geometry;
       
-      // Create vertices indices array depending on geometry class
+      // Create vertices indices array according to geometry class
       int [] verticesIndices = null;
       int [] stripVertexCount = null;
       if (geometryArray instanceof IndexedGeometryArray) {
@@ -752,7 +766,7 @@ public class PhotoRenderer {
           }
         }
 
-        // Export lines, triangles or quadrilaterals depending on the geometry
+        // Export lines, triangles or quadrilaterals according to the geometry
         if (geometryArray instanceof IndexedGeometryArray) {
           int [] normalsIndices = normals != null
               ? new int [verticesIndices.length]
@@ -821,7 +835,7 @@ public class PhotoRenderer {
           if (normalsIndices != null && !Arrays.equals(verticesIndices, normalsIndices)
               || uvsIndices != null && !Arrays.equals(verticesIndices, uvsIndices)) {
             // Remove indirection in verticesIndices, normals and uvsIndices
-            // because SunFlow use only verticesIndices
+            // because SunFlow uses only verticesIndices
             float [] directVertices = new float [verticesIndices.length * 3];
             float [] directNormals =  normalsIndices != null
                 ? new float [verticesIndices.length * 3]
@@ -988,8 +1002,8 @@ public class PhotoRenderer {
   private TexCoord2f generateTextureCoordinates(float x, float y, float z, 
                                                 Vector4f planeS, 
                                                 Vector4f planeT) {
-    return new TexCoord2f(x * planeS.x + y * planeS.y + z * planeS.z, 
-        x * planeT.x + y * planeT.y + z * planeT.z);
+    return new TexCoord2f(x * planeS.x + y * planeS.y + z * planeS.z + planeS.w, 
+        x * planeT.x + y * planeT.y + z * planeT.z + planeT.w);
   }
 
   /**
@@ -1095,7 +1109,7 @@ public class PhotoRenderer {
   }
     
   /**
-   * Stores in <code>verticesIndices</code> the indices vertexIndex1, vertexIndex2, vertexIndex3. 
+   * Stores in <code>verticesIndices</code> the indices vertexIndex1 and vertexIndex2. 
    */
   private void exportLine(GeometryArray geometryArray, 
                           int vertexIndex1, int vertexIndex2, 
@@ -1123,7 +1137,7 @@ public class PhotoRenderer {
   }
     
   /**
-   * Stores an appearance as a Sunflow shader.  
+   * Exports a Java3D appearance as a SunFlow shader.  
    */
   private void exportAppearance(Appearance appearance,
                                 String appearanceName, 
@@ -1326,8 +1340,8 @@ public class PhotoRenderer {
 
     public synchronized void imageFill(int x, int y, int width, int height, Color c, float alpha) {
       int rgba = c.copy().mul(1.0f / alpha).toNonLinear().toRGBA(alpha);
-      for (int j = 0, index = 0; j < height; j++) {
-        for (int i = 0; i < width; i++, index++) {
+      for (int j = 0; j < height; j++) {
+        for (int i = 0; i < width; i++) {
           this.image.setRGB(x + i, y + j, rgba);
         }
       }
@@ -1360,7 +1374,7 @@ public class PhotoRenderer {
   }
   
   /**
-   * A triangle used to remove faces cited for that once (opposite faces included).
+   * A triangle used to remove faces cited more that once (opposite faces included).
    */
   private static class Triangle {
     private float [] point1;
