@@ -27,8 +27,6 @@ import java.math.BigDecimal;
  * @since  1.7
  */
 public class CatalogLight extends CatalogPieceOfFurniture implements Light {
-  private static final long serialVersionUID = 1L;
-
   private final LightSource [] lightSources;
 
   /**
@@ -124,8 +122,48 @@ public class CatalogLight extends CatalogPieceOfFurniture implements Light {
                       float [][] modelRotation, String creator,
                       boolean resizable, boolean deformable, boolean texturable,
                       BigDecimal price, BigDecimal valueAddedTaxPercentage) {
-    super(id, name, description, icon, planIcon, model, width, depth, height, elevation, movable,   
-        modelRotation, creator, resizable, deformable, texturable, price, valueAddedTaxPercentage);
+    this(id, name, description, icon, planIcon, model, width, depth, height, elevation, movable,   
+        lightSources, null, modelRotation, creator, resizable, true, true, price, valueAddedTaxPercentage, null);
+  }
+         
+  /**
+   * Creates an unmodifiable catalog light of the default catalog.
+   * @param id    the id of the new light, or <code>null</code>
+   * @param name  the name of the new light
+   * @param description the description of the new light 
+   * @param icon content of the icon of the new light
+   * @param planIcon content of the icon of the new piece displayed in plan
+   * @param model content of the 3D model of the new light
+   * @param width  the width in centimeters of the new light
+   * @param depth  the depth in centimeters of the new light
+   * @param height  the height in centimeters of the new light
+   * @param elevation  the elevation in centimeters of the new light
+   * @param movable if <code>true</code>, the new light is movable
+   * @param lightSources the light sources of the new light
+   * @param staircaseCutOutShape the shape used to cut out upper levels when they intersect 
+   *            with the piece like a staircase
+   * @param modelRotation the rotation 3 by 3 matrix applied to the light model
+   * @param creator the creator of the model
+   * @param resizable if <code>true</code>, the size of the new light may be edited
+   * @param deformable if <code>true</code>, the width, depth and height of the new piece may 
+   *            change independently from each other
+   * @param texturable if <code>false</code> this piece should always keep the same color or texture.
+   * @param price the price of the new light, or <code>null</code> 
+   * @param valueAddedTaxPercentage the Value Added Tax percentage applied to the 
+   *             price of the new light or <code>null</code> 
+   * @param currency the price currency, noted with ISO 4217 code, or <code>null</code> 
+   * @since 3.4           
+   */
+  public CatalogLight(String id, String name, String description, 
+                      Content icon, Content planIcon, Content model, 
+                      float width, float depth, float height, float elevation, boolean movable, 
+                      LightSource [] lightSources, String staircaseCutOutShape,
+                      float [][] modelRotation, String creator,
+                      boolean resizable, boolean deformable, boolean texturable,
+                      BigDecimal price, BigDecimal valueAddedTaxPercentage, String currency) {
+    super(id, name, description, icon, planIcon, model, width, depth, height, elevation,   
+        movable, staircaseCutOutShape, modelRotation, creator, resizable, deformable, texturable, 
+        price, valueAddedTaxPercentage, currency);
     this.lightSources = lightSources;
   }
          
