@@ -28,8 +28,6 @@ import java.text.Collator;
  * @author Emmanuel Puybaret
  */
 public class CatalogPieceOfFurniture implements Comparable<CatalogPieceOfFurniture>, PieceOfFurniture {
-  private static final long serialVersionUID = 1L;
-
   private static final float [][] INDENTITY_ROTATION = new float [][] {{1, 0, 0}, {0, 1, 0}, {0, 0, 1}};
 
   private final String                 id;
@@ -46,6 +44,7 @@ public class CatalogPieceOfFurniture implements Comparable<CatalogPieceOfFurnitu
   private final boolean                movable;
   private final boolean                doorOrWindow;
   private final float [][]             modelRotation;
+  private final String                 staircaseCutOutShape;
   private final String                 creator;
   private final boolean                backFaceShown;
   private final Integer                color;
@@ -56,11 +55,13 @@ public class CatalogPieceOfFurniture implements Comparable<CatalogPieceOfFurnitu
   private final boolean                texturable;
   private final BigDecimal             price;
   private final BigDecimal             valueAddedTaxPercentage;
+  private final String                 currency;
 
   private FurnitureCategory            category;
 
   private static final Collator COMPARATOR = Collator.getInstance();
 
+
   /**
    * Creates a catalog piece of furniture.
    * @param name  the name of the new piece
@@ -78,8 +79,8 @@ public class CatalogPieceOfFurniture implements Comparable<CatalogPieceOfFurnitu
   public CatalogPieceOfFurniture(String name, Content icon, Content model, 
                                  float width, float depth, float height, 
                                  boolean movable, boolean doorOrWindow) {
-    this(null, name, null, icon, null, model, width, depth, height, 0, movable, doorOrWindow, null,
-        INDENTITY_ROTATION, null, false, true, true, true, null, null, (float)Math.PI / 8, true, false);
+    this(null, name, null, icon, model, width, depth, height, 0, movable, doorOrWindow, 
+        INDENTITY_ROTATION, null, true, null, null);
   }
 
   /**
@@ -110,8 +111,8 @@ public class CatalogPieceOfFurniture implements Comparable<CatalogPieceOfFurnitu
                                  boolean movable, boolean doorOrWindow, 
                                  float [][] modelRotation, String creator,
                                  boolean resizable, BigDecimal price, BigDecimal valueAddedTaxPercentage) {
-    this(id, name, description, icon, null, model, width, depth, height, elevation, movable, doorOrWindow, null,
-        modelRotation, creator, false, resizable, true, true, price, valueAddedTaxPercentage, (float)Math.PI / 8, true, false);
+    this(id, name, description, icon, model, width, depth, height, elevation, movable, 
+        modelRotation, creator, resizable, price, valueAddedTaxPercentage);
   }
          
   /**
@@ -202,12 +203,51 @@ public class CatalogPieceOfFurniture implements Comparable<CatalogPieceOfFurnitu
                                  boolean movable, float [][] modelRotation, String creator,
                                  boolean resizable, boolean deformable, boolean texturable, 
                                  BigDecimal price, BigDecimal valueAddedTaxPercentage) {
-    this(id, name, description, icon, planIcon, model, width, depth, height, elevation, movable, false, null,
-        modelRotation, creator, false, resizable, deformable, texturable,
-        price, valueAddedTaxPercentage, (float)Math.PI / 8, true, false);
+    this(id, name, description, icon, planIcon, model, width, depth, height, elevation, 
+        movable, null, modelRotation, creator, resizable, deformable, texturable,
+        price, valueAddedTaxPercentage, null);
   }
   
   /**
+   * Creates an unmodifiable catalog piece of furniture of the default catalog.
+   * @param id    the id of the new piece or <code>null</code>
+   * @param name  the name of the new piece
+   * @param description the description of the new piece 
+   * @param icon content of the icon of the new piece
+   * @param planIcon content of the icon of the new piece displayed in plan
+   * @param model content of the 3D model of the new piece
+   * @param width  the width in centimeters of the new piece
+   * @param depth  the depth in centimeters of the new piece
+   * @param height  the height in centimeters of the new piece
+   * @param elevation  the elevation in centimeters of the new piece
+   * @param movable if <code>true</code>, the new piece is movable
+   * @param staircaseCutOutShape the shape used to cut out upper levels when they intersect 
+   *            with the piece like a staircase
+   * @param modelRotation the rotation 3 by 3 matrix applied to the piece model
+   * @param creator the creator of the model
+   * @param resizable if <code>true</code>, the size of the new piece may be edited
+   * @param deformable if <code>true</code>, the width, depth and height of the new piece may 
+   *            change independently from each other
+   * @param texturable if <code>false</code> this piece should always keep the same color or texture.
+   * @param price the price of the new piece or <code>null</code> 
+   * @param valueAddedTaxPercentage the Value Added Tax percentage applied to the 
+   *             price of the new piece or <code>null</code> 
+   * @param currency the price currency, noted with ISO 4217 code, or <code>null</code> 
+   * @since 3.4
+   */
+  public CatalogPieceOfFurniture(String id, String name, String description, 
+                                 Content icon, Content planIcon, Content model, 
+                                 float width, float depth, float height, 
+                                 float elevation, boolean movable, String staircaseCutOutShape, 
+                                 float [][] modelRotation, String creator,
+                                 boolean resizable, boolean deformable, boolean texturable, 
+                                 BigDecimal price, BigDecimal valueAddedTaxPercentage, String currency) {
+    this(id, name, description, icon, planIcon, model, width, depth, height, elevation, movable, false, staircaseCutOutShape,
+        null, modelRotation, creator, false, resizable, deformable, texturable,
+        price, valueAddedTaxPercentage, currency, (float)Math.PI / 8, true, false);
+  }
+  
+ /**
    * Creates a modifiable catalog piece of furniture with all its values.
    * @param name  the name of the new piece
    * @param icon content of the icon of the new piece
@@ -232,8 +272,8 @@ public class CatalogPieceOfFurniture implements Comparable<CatalogPieceOfFurnitu
                                  boolean movable, boolean doorOrWindow, Integer color,
                                  float [][] modelRotation, boolean backFaceShown,
                                  float iconYaw, boolean proportional) {
-    this(null, name, null, icon, null, model, width, depth, height, elevation, movable, doorOrWindow, 
-        color, modelRotation, null, backFaceShown, true, true, true, null, null, iconYaw, proportional, true);
+    this(name, icon, model, width, depth, height, elevation, movable, 
+        color, modelRotation, backFaceShown, iconYaw, proportional);
   }
   
   /**
@@ -258,17 +298,47 @@ public class CatalogPieceOfFurniture implements Comparable<CatalogPieceOfFurnitu
                                  boolean movable, Integer color,
                                  float [][] modelRotation, boolean backFaceShown,
                                  float iconYaw, boolean proportional) {
+    this(name, icon, model, width, depth, height, elevation, movable,  
+        null, color, modelRotation, backFaceShown, iconYaw, proportional);
+  }
+  
+  /**
+   * Creates a modifiable catalog piece of furniture with all its values.
+   * @param name  the name of the new piece
+   * @param icon content of the icon of the new piece
+   * @param model content of the 3D model of the new piece
+   * @param width  the width in centimeters of the new piece
+   * @param depth  the depth in centimeters of the new piece
+   * @param height  the height in centimeters of the new piece
+   * @param elevation  the elevation in centimeters of the new piece
+   * @param movable if <code>true</code>, the new piece is movable
+   * @param staircaseCutOutShape the shape used to cut out upper levels when they intersect 
+   *            with the piece like a staircase
+   * @param color the color of the piece as RGB code or <code>null</code> if piece color is unchanged
+   * @param modelRotation the rotation 3 by 3 matrix applied to the piece model
+   * @param backFaceShown <code>true</code> if back face should be shown
+   * @param iconYaw the yaw angle used to create the piece icon
+   * @param proportional if <code>true</code>, size proportions will be kept
+   * @since 3.4 
+   */
+  public CatalogPieceOfFurniture(String name, Content icon, Content model, 
+                                 float width, float depth, float height, float elevation, 
+                                 boolean movable, String staircaseCutOutShape,
+                                 Integer color, float [][] modelRotation, 
+                                 boolean backFaceShown,
+                                 float iconYaw, boolean proportional) {
     this(null, name, null, icon, null, model, width, depth, height, elevation, movable, false, 
-        color, modelRotation, null, backFaceShown, true, true, true, null, null, iconYaw, proportional, true);
+        staircaseCutOutShape, color, modelRotation, null, backFaceShown, true, true, true, null, null, null, iconYaw, proportional, true);
   }
   
   private CatalogPieceOfFurniture(String id, String name, String description, 
                                   Content icon, Content planIcon, Content model, 
                                   float width, float depth, float height, float elevation, 
                                   boolean movable, boolean doorOrWindow, 
-                                  Integer color, float [][] modelRotation, String creator, boolean backFaceShown,
+                                  String staircaseCutOutShape, Integer color, float [][] modelRotation,
+                                  String creator, boolean backFaceShown,
                                   boolean resizable, boolean deformable, boolean texturable,
-                                  BigDecimal price, BigDecimal valueAddedTaxPercentage,
+                                  BigDecimal price, BigDecimal valueAddedTaxPercentage, String currency,
                                   float iconYaw, boolean proportional, boolean modifiable) {
     this.id = id;
     this.name = name;
@@ -283,9 +353,11 @@ public class CatalogPieceOfFurniture implements Comparable<CatalogPieceOfFurnitu
     this.movable = movable;
     this.doorOrWindow = doorOrWindow;
     this.color = color;
+    this.staircaseCutOutShape = staircaseCutOutShape;
     this.creator = creator;
     this.price = price;
     this.valueAddedTaxPercentage = valueAddedTaxPercentage;
+    this.currency = currency;
     if (modelRotation == null) {
       this.modelRotation = INDENTITY_ROTATION;
     } else {
@@ -404,6 +476,15 @@ public class CatalogPieceOfFurniture implements Comparable<CatalogPieceOfFurnitu
                            {modelRotation [2][0], modelRotation [2][1], modelRotation [2][2]}};
   }
 
+  /**
+   * Returns the shape used to cut out upper levels when they intersect with the piece   
+   * like a staircase.
+   * @since 3.4
+   */
+  public String getStaircaseCutOutShape() {
+    return this.staircaseCutOutShape;
+  }
+  
   /**
    * Returns the creator of this piece.
    */
@@ -484,6 +565,15 @@ public class CatalogPieceOfFurniture implements Comparable<CatalogPieceOfFurnitu
     return this.valueAddedTaxPercentage;
   }
 
+  /**
+   * Returns the price currency, noted with ISO 4217 code, or <code>null</code> 
+   * if it has no price or default currency should be used.
+   * @since 3.4
+   */
+  public String getCurrency() {
+    return this.currency;
+  }
+
   /**
    * Returns the category of this piece of furniture.
    */
