@@ -41,7 +41,7 @@ import java.util.Map;
  * A piece of furniture in {@linkplain Home home}.
  * @author Emmanuel Puybaret
  */
-public class HomePieceOfFurniture implements PieceOfFurniture, Serializable, Selectable {
+public class HomePieceOfFurniture implements PieceOfFurniture, Serializable, Selectable, Elevatable {
   private static final long serialVersionUID = 1L;
   
   private static final double TWICE_PI = 2 * Math.PI;
@@ -51,14 +51,14 @@ public class HomePieceOfFurniture implements PieceOfFurniture, Serializable, Sel
    * to a piece of furniture will be notified under a property name equal to the string value of one these properties.
    */
   public enum Property {NAME, NAME_VISIBLE, NAME_X_OFFSET, NAME_Y_OFFSET, NAME_STYLE,
-      DESCRIPTION, WIDTH, DEPTH, HEIGHT, COLOR, TEXTURE, SHININESS, VISIBLE, X, Y, ELEVATION, ANGLE, MODEL_MIRRORED, MOVABLE};
+      DESCRIPTION, WIDTH, DEPTH, HEIGHT, COLOR, TEXTURE, SHININESS, VISIBLE, X, Y, ELEVATION, ANGLE, MODEL_MIRRORED, MOVABLE, LEVEL};
   
   /** 
    * The properties on which home furniture may be sorted.  
    */
   public enum SortableProperty {CATALOG_ID, NAME, WIDTH, DEPTH, HEIGHT, MOVABLE, 
                                 DOOR_OR_WINDOW, COLOR, TEXTURE, VISIBLE, X, Y, ELEVATION, ANGLE,
-                                PRICE, VALUE_ADDED_TAX, VALUE_ADDED_TAX_PERCENTAGE, PRICE_VALUE_ADDED_TAX_INCLUDED};
+                                PRICE, VALUE_ADDED_TAX, VALUE_ADDED_TAX_PERCENTAGE, PRICE_VALUE_ADDED_TAX_INCLUDED, LEVEL};
   private static final Map<SortableProperty, Comparator<HomePieceOfFurniture>> SORTABLE_PROPERTY_COMPARATORS;
   private static final float [][] IDENTITY = new float [][] {{1, 0, 0}, {0, 1, 0}, {0, 0, 1}};
   
@@ -154,6 +154,11 @@ public class HomePieceOfFurniture implements PieceOfFurniture, Serializable, Sel
           return HomePieceOfFurniture.compare(piece1.angle, piece2.angle);
         }
       });
+    SORTABLE_PROPERTY_COMPARATORS.put(SortableProperty.LEVEL, new Comparator<HomePieceOfFurniture>() {
+        public int compare(HomePieceOfFurniture piece1, HomePieceOfFurniture piece2) {
+          return HomePieceOfFurniture.compare(piece1.getLevel(), piece2.getLevel());
+        }
+      });
     SORTABLE_PROPERTY_COMPARATORS.put(SortableProperty.PRICE, new Comparator<HomePieceOfFurniture>() {
         public int compare(HomePieceOfFurniture piece1, HomePieceOfFurniture piece2) {
           return HomePieceOfFurniture.compare(piece1.price, piece2.price);
@@ -177,10 +182,7 @@ public class HomePieceOfFurniture implements PieceOfFurniture, Serializable, Sel
   }
   
   private static int compare(float value1, float value2) {
-    return value1 < value2 
-               ? -1
-               : (value1 == value2
-                   ? 0 : 1);
+    return Float.compare(value1, value2);
   }
   
   private static int compare(boolean value1, boolean value2) {
@@ -199,6 +201,16 @@ public class HomePieceOfFurniture implements PieceOfFurniture, Serializable, Sel
     }
   }
   
+  private static int compare(Level level1, Level level2) {
+    if (level1 == null) {
+      return -1;
+    } else if (level2 == null) {
+      return 1; 
+    } else {
+      return Float.compare(level1.getElevation(), level2.getElevation());
+    }
+  }
+  
   private String                 catalogId;
   private String                 name;
   private boolean                nameVisible;
@@ -219,17 +231,20 @@ public class HomePieceOfFurniture implements PieceOfFurniture, Serializable, Sel
   private HomeTexture            texture;
   private Float                  shininess;
   private float [][]             modelRotation;
+  private String                 staircaseCutOutShape;
   private boolean                backFaceShown;
   private boolean                resizable;
   private boolean                deformable;
   private boolean                texturable;
   private BigDecimal             price;
   private BigDecimal             valueAddedTaxPercentage;
+  private String                 currency;
   private boolean                visible;
   private float                  x;
   private float                  y;
   private float                  angle;
   private boolean                modelMirrored;
+  private Level                  level;
 
   private transient PropertyChangeSupport propertyChangeSupport = new PropertyChangeSupport(this);
   private transient Shape shapeCache;
@@ -252,12 +267,14 @@ public class HomePieceOfFurniture implements PieceOfFurniture, Serializable, Sel
     this.doorOrWindow = piece.isDoorOrWindow();
     this.color = piece.getColor();
     this.modelRotation = piece.getModelRotation();
+    this.staircaseCutOutShape = piece.getStaircaseCutOutShape();
     this.backFaceShown = piece.isBackFaceShown();
     this.resizable = piece.isResizable();
     this.deformable = piece.isDeformable();
     this.texturable = piece.isTexturable();
     this.price = piece.getPrice();
     this.valueAddedTaxPercentage = piece.getValueAddedTaxPercentage();
+    this.currency = piece.getCurrency();
     if (piece instanceof HomePieceOfFurniture) {
       HomePieceOfFurniture homePiece = 
           (HomePieceOfFurniture)piece;
@@ -509,14 +526,27 @@ public class HomePieceOfFurniture implements PieceOfFurniture, Serializable, Sel
   }
 
   /**
-   * Returns the elevation of the bottom of this piece of furniture. 
+   * Returns the elevation of the bottom of this piece of furniture on its level. 
    */
   public float getElevation() {
     return this.elevation;
   }
 
   /**
-   * Sets the elevation of this piece of furniture. Once this piece is updated, 
+   * Returns the elevation of the bottom of this piece of furniture 
+   * from the ground according to the elevation of its level.
+   * @since 3.4 
+   */
+  public float getGroundElevation() {
+    if (this.level != null) {
+      return this.elevation + this.level.getElevation();
+    } else {
+      return this.elevation;
+    }
+  }
+
+  /**
+   * Sets the elevation of this piece of furniture on its level. Once this piece is updated, 
    * listeners added to this piece will receive a change notification.
    */
   public void setElevation(float elevation) {
@@ -719,6 +749,15 @@ public class HomePieceOfFurniture implements PieceOfFurniture, Serializable, Sel
     }
   }
 
+  /**
+   * Returns the price currency, noted with ISO 4217 code, or <code>null</code> 
+   * if it has no price or default currency should be used.
+   * @since 3.4
+   */
+  public String getCurrency() {
+    return this.currency;
+  }
+
   /**
    * Returns <code>true</code> if this piece of furniture is visible.
    */
@@ -834,6 +873,15 @@ public class HomePieceOfFurniture implements PieceOfFurniture, Serializable, Sel
                            {this.modelRotation[2][0], this.modelRotation[2][1], this.modelRotation[2][2]}};
   }
 
+  /**
+   * Returns the shape used to cut out upper levels when they intersect with the piece   
+   * like a staircase.
+   * @since 3.4
+   */
+  public String getStaircaseCutOutShape() {
+    return this.staircaseCutOutShape;
+  }
+
   /**
    * Returns <code>true</code> if the back face of the piece of furniture
    * model should be displayed.
@@ -841,6 +889,38 @@ public class HomePieceOfFurniture implements PieceOfFurniture, Serializable, Sel
   public boolean isBackFaceShown() {
     return this.backFaceShown;
   }
+
+  /**
+   * Returns the level which this piece belongs to. 
+   * @since 3.4
+   */
+  public Level getLevel() {
+    return this.level;
+  }
+
+  /**
+   * Sets the level of this piece of furniture. Once this piece is updated, 
+   * listeners added to this piece will receive a change notification.
+   * @since 3.4
+   */
+  public void setLevel(Level level) {
+    if (level != this.level) {
+      Level oldLevel = this.level;
+      this.level = level;
+      this.propertyChangeSupport.firePropertyChange(Property.LEVEL.name(), oldLevel, level);
+    }
+  }
+  
+  /**
+   * Returns <code>true</code> if this piece is visible at the given level.
+   * @since 3.4
+   */
+  public boolean isAtLevel(Level level) {
+    return this.level == level 
+        || this.level != null && level != null
+            && this.level.getElevation() < level.getElevation()
+            && this.level.getElevation() + this.elevation + this.height > level.getElevation();
+  }
   
   /**
    * Returns the points of each corner of a piece.
@@ -994,6 +1074,7 @@ public class HomePieceOfFurniture implements PieceOfFurniture, Serializable, Sel
     try {
       HomePieceOfFurniture clone = (HomePieceOfFurniture)super.clone();
       clone.propertyChangeSupport = new PropertyChangeSupport(clone);
+      clone.level = null;
       return clone;
     } catch (CloneNotSupportedException ex) {
       throw new IllegalStateException("Super class isn't cloneable"); 
