@@ -32,6 +32,7 @@ public class HomeTexture implements TextureImage, Serializable {
   private final Content image;
   private final float width;
   private final float height;
+  private final boolean leftToRightOriented; 
   
   /**
    * Creates a home texture from an existing one.
@@ -42,6 +43,9 @@ public class HomeTexture implements TextureImage, Serializable {
     this.image = texture.getImage();
     this.width = texture.getWidth();
     this.height = texture.getHeight();
+    // Texture is left to right oriented when applied on objects seen from front
+    // added to homes with a version 3.4 and higher
+    this.leftToRightOriented = true; 
   }
   
   /**
@@ -72,6 +76,14 @@ public class HomeTexture implements TextureImage, Serializable {
     return this.height;
   }
 
+  /**
+   * Returns <code>true</code> if the objects using this texture should take into account 
+   * the orientation of the texture.
+   */
+  public boolean isLeftToRightOriented() {
+    return this.leftToRightOriented;
+  }
+  
   /**
    * Returns <code>true</code> if the object in parameter is equal to this texture.
    */
@@ -84,7 +96,8 @@ public class HomeTexture implements TextureImage, Serializable {
       return texture.name.equals(this.name)
           && texture.image.equals(this.image)
           && texture.width == this.width
-          && texture.height == this.height;
+          && texture.height == this.height
+          && texture.leftToRightOriented == this.leftToRightOriented;
     } else {
       return false;
     }
