@@ -72,12 +72,13 @@ import com.eteks.sweethome3d.model.Content;
 import com.eteks.sweethome3d.model.Home;
 import com.eteks.sweethome3d.model.HomeFurnitureGroup;
 import com.eteks.sweethome3d.model.HomePieceOfFurniture;
+import com.eteks.sweethome3d.model.HomePieceOfFurniture.SortableProperty;
 import com.eteks.sweethome3d.model.HomeTexture;
+import com.eteks.sweethome3d.model.Level;
 import com.eteks.sweethome3d.model.Selectable;
 import com.eteks.sweethome3d.model.SelectionEvent;
 import com.eteks.sweethome3d.model.SelectionListener;
 import com.eteks.sweethome3d.model.UserPreferences;
-import com.eteks.sweethome3d.model.HomePieceOfFurniture.SortableProperty;
 import com.eteks.sweethome3d.tools.ResourceURLContent;
 import com.eteks.sweethome3d.viewcontroller.FurnitureController;
 import com.eteks.sweethome3d.viewcontroller.View;
@@ -242,7 +243,25 @@ public class FurnitureTable extends JTable implements View, Printable {
     addMouseListener(new MouseAdapter () {
         @Override
         public void mouseClicked(MouseEvent ev) {
-          if (ev.getClickCount() == 2) {
+          int column = columnAtPoint(ev.getPoint());
+          int row = rowAtPoint(ev.getPoint());
+          boolean isVisibleColumn = false;
+          if (column >= 0
+              && row >= 0
+              && getColumnModel().getColumn(column).getIdentifier() == HomePieceOfFurniture.SortableProperty.VISIBLE) {
+            Component visibilityComponent = getCellRenderer(row, column).
+                getTableCellRendererComponent(FurnitureTable.this, getValueAt(row, column), false, false, row, column);
+            Rectangle cellRect = getCellRect(row, column, false);
+            // Center visibilityComponent in cell rect
+            visibilityComponent.setSize(visibilityComponent.getPreferredSize());
+            visibilityComponent.setLocation(cellRect.x + (cellRect.width - visibilityComponent.getWidth()) / 2, 
+                cellRect.y + (cellRect.height - visibilityComponent.getHeight()) / 2);
+            // Check if mouse point is exactly on the visibility component
+            isVisibleColumn = visibilityComponent.getBounds().contains(ev.getPoint());
+          }
+          if (isVisibleColumn) {
+            controller.toggleSelectedFurnitureVisibility();
+          } else if (ev.getClickCount() == 2) {
             controller.modifySelectedFurniture();
           }
         }
@@ -302,7 +321,7 @@ public class FurnitureTable extends JTable implements View, Printable {
     home.addPropertyChangeListener(Home.Property.FURNITURE_SORTED_PROPERTY, sortListener);
     home.addPropertyChangeListener(Home.Property.FURNITURE_DESCENDING_SORTED, sortListener);
     
-    final PropertyChangeListener furnitureChangeListener = 
+    final PropertyChangeListener changeListener = 
       new PropertyChangeListener () {
         public void propertyChange(PropertyChangeEvent ev) {
           // As furniture properties values change may alter sort order, update sort and whole table
@@ -312,14 +331,26 @@ public class FurnitureTable extends JTable implements View, Printable {
         }
       };
     for (HomePieceOfFurniture piece : home.getFurniture()) {
-      piece.addPropertyChangeListener(furnitureChangeListener);
+      piece.addPropertyChangeListener(changeListener);
     }
     home.addFurnitureListener(new CollectionListener<HomePieceOfFurniture>() {
-        public void collectionChanged(CollectionEvent<HomePieceOfFurniture> ev) {
+      public void collectionChanged(CollectionEvent<HomePieceOfFurniture> ev) {
           if (ev.getType() == CollectionEvent.Type.ADD) {
-            ev.getItem().addPropertyChangeListener(furnitureChangeListener);
+            ev.getItem().addPropertyChangeListener(changeListener);
           } else {
-            ev.getItem().removePropertyChangeListener(furnitureChangeListener);
+            ev.getItem().removePropertyChangeListener(changeListener);
+          }
+        }
+      });
+    for (Level level : home.getLevels()) {
+      level.addPropertyChangeListener(changeListener);
+    }
+    home.addLevelsListener(new CollectionListener<Level>() {
+        public void collectionChanged(CollectionEvent<Level> ev) {
+          if (ev.getType() == CollectionEvent.Type.ADD) {
+            ev.getItem().addPropertyChangeListener(changeListener);
+          } else {
+            ev.getItem().removePropertyChangeListener(changeListener);
           }
         }
       });
@@ -552,6 +583,12 @@ public class FurnitureTable extends JTable implements View, Printable {
               // Copy piece name
               csv.append(copiedPiece.getName());
               break;
+            case LEVEL :
+              // Copy level name
+              csv.append(copiedPiece.getLevel() != null 
+                  ? copiedPiece.getLevel().getName() 
+                  : "");
+              break;
             case COLOR :
               if (copiedPiece.getColor() != null) {
                 // Copy piece color at #xxxxxx format              
@@ -611,6 +648,14 @@ public class FurnitureTable extends JTable implements View, Printable {
     tableModel.setFurnitureFilter(filter);
   }
   
+  /**
+   * Returns the filter applied to the furniture displayed in this table.
+   */
+  public FurnitureTable.FurnitureFilter getFurnitureFilter() {
+    FurnitureTableModel tableModel = (FurnitureTableModel)getModel();
+    return tableModel.getFurnitureFilter();
+  }
+  
   /**
    * Column table model used by this table.
    */
@@ -751,6 +796,8 @@ public class FurnitureTable extends JTable implements View, Printable {
           return preferences.getLocalizedString(FurnitureTable.class, "elevationColumn");
         case ANGLE :
           return preferences.getLocalizedString(FurnitureTable.class, "angleColumn");
+        case LEVEL :
+          return preferences.getLocalizedString(FurnitureTable.class, "levelColumn");
         case COLOR :
           return preferences.getLocalizedString(FurnitureTable.class, "colorColumn");
         case TEXTURE :
@@ -791,6 +838,8 @@ public class FurnitureTable extends JTable implements View, Printable {
           return 50;
         case ANGLE :
           return 35;        
+        case LEVEL :
+          return 70;        
         case COLOR :
         case TEXTURE :
           return 30;        
@@ -832,6 +881,8 @@ public class FurnitureTable extends JTable implements View, Printable {
           return getSizeRenderer(HomePieceOfFurniture.SortableProperty.ELEVATION, preferences);
         case ANGLE :
           return getAngleRenderer();        
+        case LEVEL :
+          return getLevelRenderer();        
         case COLOR :
           return getColorRenderer();        
         case TEXTURE :
@@ -871,6 +922,22 @@ public class FurnitureTable extends JTable implements View, Printable {
       };
     }
 
+    /**
+     * Returns a renderer that displays the level name of a piece of furniture. 
+     */
+    private TableCellRenderer getLevelRenderer() {
+      return new DefaultTableCellRenderer() { 
+        @Override
+        public Component getTableCellRendererComponent(JTable table, 
+             Object value, boolean isSelected, boolean hasFocus, 
+             int row, int column) {
+          HomePieceOfFurniture piece = (HomePieceOfFurniture)value; 
+          Level level = piece.getLevel();
+          return super.getTableCellRendererComponent(
+              table, level != null  ? level.getName()  : "", isSelected, hasFocus, row, column); 
+        }
+      };
+    }
     /**
      * Returns a renderer that displays the name of a piece of furniture with its icon ahead. 
      */
@@ -984,13 +1051,14 @@ public class FurnitureTable extends JTable implements View, Printable {
       // Renderer super class used to display sizes
       class PriceRenderer extends DefaultTableCellRenderer {
         public Component getTableCellRendererComponent(JTable table, 
-             Object value, boolean isSelected, boolean hasFocus, 
+             BigDecimal price, String currency, boolean isSelected, boolean hasFocus, 
              int row, int column) {
-          String currency = preferences.getCurrency();
-          if (value != null && currency != null) {
+          String defaultCurrency = preferences.getCurrency();
+          String value;
+          if (price != null && defaultCurrency != null) {
             NumberFormat currencyFormat = DecimalFormat.getCurrencyInstance();
-            currencyFormat.setCurrency(Currency.getInstance(currency));
-            value = currencyFormat.format((BigDecimal)value);
+            currencyFormat.setCurrency(Currency.getInstance(currency != null ? currency : defaultCurrency));
+            value = currencyFormat.format(price);
           } else {
             value = "";
           }
@@ -1006,8 +1074,9 @@ public class FurnitureTable extends JTable implements View, Printable {
               @Override
               public Component getTableCellRendererComponent(JTable table, 
                   Object value, boolean isSelected, boolean hasFocus, int row, int column) {
+                HomePieceOfFurniture piece = (HomePieceOfFurniture)value;
                 return super.getTableCellRendererComponent(table, 
-                    ((HomePieceOfFurniture)value).getPrice(), isSelected, hasFocus, row, column);
+                    piece.getPrice(), piece.getCurrency(), isSelected, hasFocus, row, column);
               }
             };
         case VALUE_ADDED_TAX :
@@ -1015,8 +1084,9 @@ public class FurnitureTable extends JTable implements View, Printable {
               @Override
               public Component getTableCellRendererComponent(JTable table, 
                   Object value, boolean isSelected, boolean hasFocus, int row, int column) {
+                HomePieceOfFurniture piece = (HomePieceOfFurniture)value;
                 return super.getTableCellRendererComponent(table, 
-                    ((HomePieceOfFurniture)value).getValueAddedTax(), isSelected, hasFocus, row, column);
+                    piece.getValueAddedTax(), piece.getCurrency(), isSelected, hasFocus, row, column);
               }
             };
         case PRICE_VALUE_ADDED_TAX_INCLUDED :
@@ -1024,8 +1094,9 @@ public class FurnitureTable extends JTable implements View, Printable {
               @Override
               public Component getTableCellRendererComponent(JTable table, 
                   Object value, boolean isSelected, boolean hasFocus, int row, int column) {
+                HomePieceOfFurniture piece = (HomePieceOfFurniture)value;
                 return super.getTableCellRendererComponent(table, 
-                    ((HomePieceOfFurniture)value).getPriceValueAddedTaxIncluded(), isSelected, hasFocus, row, column);
+                    piece.getPriceValueAddedTaxIncluded(), piece.getCurrency(), isSelected, hasFocus, row, column);
               }
             };
         default :
@@ -1163,20 +1234,27 @@ public class FurnitureTable extends JTable implements View, Printable {
       // Renderer super class used to display booleans
       class BooleanRenderer implements TableCellRenderer {
         private TableCellRenderer booleanRenderer;
+        private final boolean enabled;
+
+        public BooleanRenderer(boolean enabled) {
+          this.enabled = enabled;
+        }
 
         public Component getTableCellRendererComponent(JTable table, 
              Object value, boolean isSelected, boolean hasFocus, int row, int column) {
           if (this.booleanRenderer == null) {
             this.booleanRenderer = table.getDefaultRenderer(Boolean.class);
           }
-          return this.booleanRenderer.getTableCellRendererComponent(
+          Component component = this.booleanRenderer.getTableCellRendererComponent(
               table, value, isSelected, hasFocus, row, column);
+          component.setEnabled(this.enabled);
+          return component;
         }
       };
       
       switch (property) {
         case MOVABLE :
-          return new BooleanRenderer() {
+          return new BooleanRenderer(false) {
               @Override
               public Component getTableCellRendererComponent(JTable table, 
                   Object value, boolean isSelected, boolean hasFocus, int row, int column) {
@@ -1185,7 +1263,7 @@ public class FurnitureTable extends JTable implements View, Printable {
               }
             };
         case DOOR_OR_WINDOW :
-          return new BooleanRenderer() {
+          return new BooleanRenderer(false) {
               @Override
               public Component getTableCellRendererComponent(JTable table, 
                   Object value, boolean isSelected, boolean hasFocus, int row, int column) {
@@ -1194,7 +1272,7 @@ public class FurnitureTable extends JTable implements View, Printable {
               }
             };
         case VISIBLE :
-          return new BooleanRenderer() {
+          return new BooleanRenderer(true) {
               @Override
               public Component getTableCellRendererComponent(JTable table, 
                   Object value, boolean isSelected, boolean hasFocus, int row, int column) {
@@ -1408,6 +1486,13 @@ public class FurnitureTable extends JTable implements View, Printable {
       this.furnitureFilter = furnitureFilter;      
       filterAndSortFurniture();
     }
+    
+    /**
+     * Returns the filter applied to the furniture listed in this model.
+     */
+    public FurnitureFilter getFurnitureFilter() {
+      return this.furnitureFilter;
+    }
   }
   
   /**
