@@ -44,8 +44,10 @@ import javax.swing.JComponent;
 import javax.swing.JLabel;
 
 import com.eteks.sweethome3d.model.Home;
+import com.eteks.sweethome3d.model.HomePieceOfFurniture;
 import com.eteks.sweethome3d.model.HomePrint;
 import com.eteks.sweethome3d.model.LengthUnit;
+import com.eteks.sweethome3d.model.Level;
 import com.eteks.sweethome3d.viewcontroller.ContentManager;
 import com.eteks.sweethome3d.viewcontroller.HomeController;
 import com.eteks.sweethome3d.viewcontroller.PlanView;
@@ -302,8 +304,31 @@ public class HomePrintableComponent extends JComponent implements Printable {
     View furnitureView = this.controller.getFurnitureController().getView();
     if (furnitureView != null 
         && (homePrint == null || homePrint.isFurniturePrinted())) {
-      // Try to print next furniture view page
+      FurnitureTable furnitureTable = null;
+      final FurnitureTable.FurnitureFilter furnitureFilter;
+      if (furnitureView instanceof FurnitureTable
+          && (homePrint == null
+              || homePrint.isPlanPrinted()
+              || homePrint.isView3DPrinted())) {
+        final Level selectedLevel = home.getSelectedLevel();
+        furnitureTable = (FurnitureTable)furnitureView;
+        furnitureFilter = furnitureTable.getFurnitureFilter();
+        furnitureTable.setFurnitureFilter(new FurnitureTable.FurnitureFilter() {
+            public boolean include(Home home, HomePieceOfFurniture piece) {
+              // Print only furniture at selected level when the plan or the 3D view is printed
+              return (furnitureFilter == null || furnitureFilter.include(home, piece))
+                  && piece.isAtLevel(selectedLevel);
+            }
+          });
+      } else {
+        furnitureFilter = null;
+      }
+      // Try to print next furniture view page      
       pageExists = ((Printable)furnitureView).print(g2D, pageFormat, page);
+      if (furnitureTable != null) {
+        // Restore previous filter
+        ((FurnitureTable)furnitureView).setFurnitureFilter(furnitureFilter);
+      }
       if (pageExists == PAGE_EXISTS) {
         this.furniturePageCount++;
       }
