@@ -28,6 +28,8 @@ import java.awt.Graphics2D;
 import java.awt.GridBagConstraints;
 import java.awt.GridBagLayout;
 import java.awt.Insets;
+import java.awt.Point;
+import java.awt.Rectangle;
 import java.awt.Window;
 import java.awt.datatransfer.DataFlavor;
 import java.awt.datatransfer.Transferable;
@@ -875,6 +877,7 @@ public class PhotoPanel extends JPanel implements DialogView {
       dialog.setModal(false);
      
       Component homeRoot = SwingUtilities.getRoot((Component)parentView);
+      Point dialogLocation = null;
       if (homeRoot != null) {
         // Restore location if it exists
         Integer x = (Integer)this.home.getVisualProperty(PHOTO_DIALOG_X_VISUAL_PROPERTY);
@@ -899,16 +902,19 @@ public class PhotoPanel extends JPanel implements DialogView {
         if (x != null && y != null 
             && x + dialogWidth <= screenRightBorder
             && y + dialogHeight <= screenBottomBorder) {
-          dialog.setLocation(x, y);
+          dialogLocation = new Point(x, y);
         } else if (screenRightBorder - windowRightBorder > dialogWidth / 2
                    || dialogHeight == screenHeight) {
           // If there some space left at the right of the window,
           // move the dialog to the right of window
-          dialog.setLocation(Math.min(windowRightBorder + 5, screenRightBorder - dialogWidth), 
+          dialogLocation = new Point(Math.min(windowRightBorder + 5, screenRightBorder - dialogWidth), 
               Math.max(Math.min(homeRoot.getY(), screenSize.height - dialogHeight - screenInsets.bottom), screenInsets.top));
-        } else {
-          dialog.setLocationByPlatform(true);
         }
+      }
+      if (dialogLocation != null
+          && SwingTools.isRectangleVisibleAtScreen(new Rectangle(dialogLocation, dialog.getSize()))) {
+        dialog.setLocationByPlatform(false);
+        dialog.setLocation(dialogLocation);
       } else {
         dialog.setLocationByPlatform(true);
       }
