@@ -36,10 +36,16 @@ import javax.swing.undo.UndoableEditSupport;
 
 import com.eteks.sweethome3d.model.CollectionEvent;
 import com.eteks.sweethome3d.model.CollectionListener;
+import com.eteks.sweethome3d.model.DoorOrWindow;
 import com.eteks.sweethome3d.model.Home;
+import com.eteks.sweethome3d.model.HomeDoorOrWindow;
 import com.eteks.sweethome3d.model.HomeFurnitureGroup;
+import com.eteks.sweethome3d.model.HomeLight;
 import com.eteks.sweethome3d.model.HomePieceOfFurniture;
 import com.eteks.sweethome3d.model.HomePieceOfFurniture.SortableProperty;
+import com.eteks.sweethome3d.model.Level;
+import com.eteks.sweethome3d.model.Light;
+import com.eteks.sweethome3d.model.PieceOfFurniture;
 import com.eteks.sweethome3d.model.Selectable;
 import com.eteks.sweethome3d.model.SelectionEvent;
 import com.eteks.sweethome3d.model.SelectionListener;
@@ -166,8 +172,9 @@ public class FurnitureController implements Controller {
       basePlanLocked &= !isPieceOfFurniturePartOfBasePlan(newFurniture [i]);
     }  
     final boolean newBasePlanLocked = basePlanLocked;
+    final Level furnitureLevel = this.home.getSelectedLevel();
     
-    doAddFurniture(newFurniture, furnitureIndex, newBasePlanLocked); 
+    doAddFurniture(newFurniture, furnitureIndex, furnitureLevel, null, newBasePlanLocked); 
     if (this.undoSupport != null) {
       UndoableEdit undoableEdit = new AbstractUndoableEdit() {
         @Override
@@ -180,7 +187,7 @@ public class FurnitureController implements Controller {
         @Override
         public void redo() throws CannotRedoException {
           super.redo();
-          doAddFurniture(newFurniture, furnitureIndex, newBasePlanLocked); 
+          doAddFurniture(newFurniture, furnitureIndex, furnitureLevel, null, newBasePlanLocked); 
         }
         
         @Override
@@ -194,9 +201,12 @@ public class FurnitureController implements Controller {
   
   private void doAddFurniture(HomePieceOfFurniture [] furniture,
                               int [] furnitureIndex,
+                              Level furnitureLevel, 
+                              Level [] furnitureLevels, 
                               boolean basePlanLocked) {
     for (int i = 0; i < furnitureIndex.length; i++) {
       this.home.addPieceOfFurniture (furniture [i], furnitureIndex [i]);
+      furniture [i].setLevel(furnitureLevels != null ? furnitureLevels [i] : furnitureLevel);
     }
     this.home.setBasePlanLocked(basePlanLocked);
     this.home.setSelectedItems(Arrays.asList(furniture)); 
@@ -228,9 +238,12 @@ public class FurnitureController implements Controller {
     final HomePieceOfFurniture [] furniture = sortedMap.values().
         toArray(new HomePieceOfFurniture [sortedMap.size()]); 
     final int [] furnitureIndex = new int [furniture.length];
+    final Level [] furnitureLevels = new Level [furniture.length];
     int i = 0;
     for (int index : sortedMap.keySet()) {
-      furnitureIndex [i++] = index; 
+      furnitureIndex [i] = index; 
+      furnitureLevels [i] = furniture [i].getLevel();
+      i++;
     }
     doDeleteFurniture(furniture, basePlanLocked); 
     if (this.undoSupport != null) {
@@ -238,7 +251,7 @@ public class FurnitureController implements Controller {
         @Override
         public void undo() throws CannotUndoException {
           super.undo();
-          doAddFurniture(furniture, furnitureIndex, basePlanLocked); 
+          doAddFurniture(furniture, furnitureIndex, null, furnitureLevels, basePlanLocked); 
         }
         
         @Override
@@ -305,6 +318,19 @@ public class FurnitureController implements Controller {
     return true;
   }
 
+  /**
+   * Returns a new home piece of furniture created from an other given <code>piece</code> of furniture.
+   */
+  public HomePieceOfFurniture createHomePieceOfFurniture(PieceOfFurniture piece) {
+    if (piece instanceof DoorOrWindow) {
+      return new HomeDoorOrWindow((DoorOrWindow)piece);
+    } else if (piece instanceof Light) {
+      return new HomeLight((Light)piece);
+    } else {
+      return new HomePieceOfFurniture(piece);
+    }
+  }
+
   /**
    * Returns the furniture among the given list that are not part of the base plan.
    */
@@ -398,6 +424,7 @@ public class FurnitureController implements Controller {
               HomePieceOfFurniture.SortableProperty.Y,
               HomePieceOfFurniture.SortableProperty.ELEVATION,
               HomePieceOfFurniture.SortableProperty.ANGLE,
+              HomePieceOfFurniture.SortableProperty.LEVEL,
               HomePieceOfFurniture.SortableProperty.COLOR,
               HomePieceOfFurniture.SortableProperty.TEXTURE,
               HomePieceOfFurniture.SortableProperty.MOVABLE,
@@ -437,6 +464,18 @@ public class FurnitureController implements Controller {
     }
   }
 
+  /**
+   * Controls the modification of the visibility of the selected piece of furniture.
+   */
+  public void toggleSelectedFurnitureVisibility() {
+    if (Home.getFurnitureSubList(this.home.getSelectedItems()).size() == 1) {
+      HomeFurnitureController controller = new HomeFurnitureController(this.home, this.preferences,  
+          this.viewFactory, this.contentManager, this.undoSupport);
+      controller.setVisible(!controller.getVisible());
+      controller.modifyFurniture();
+    }
+  }
+
   /**
    * Groups the selected furniture as one piece of furniture.
    */
@@ -455,31 +494,49 @@ public class FurnitureController implements Controller {
       final HomePieceOfFurniture [] groupPieces = sortedMap.values().
           toArray(new HomePieceOfFurniture [sortedMap.size()]); 
       final int [] groupPiecesIndex = new int [groupPieces.length];
+      final Level [] groupPiecesLevel = new Level [groupPieces.length];
+      final float [] groupPiecesElevation = new float [groupPieces.length];
       final boolean [] groupPiecesMovable = new boolean [groupPieces.length];
       final boolean [] groupPiecesVisible = new boolean [groupPieces.length];
+      Level minLevel = this.home.getSelectedLevel();
       int i = 0;
       for (Entry<Integer, HomePieceOfFurniture> pieceEntry : sortedMap.entrySet()) {
         groupPiecesIndex [i] = pieceEntry.getKey();
-        groupPiecesMovable [i] = pieceEntry.getValue().isMovable();
-        groupPiecesVisible [i++] = pieceEntry.getValue().isVisible();
+        HomePieceOfFurniture piece = pieceEntry.getValue();
+        groupPiecesLevel [i] = piece.getLevel();
+        groupPiecesElevation [i] = piece.getElevation();
+        groupPiecesMovable [i] = piece.isMovable();
+        groupPiecesVisible [i] = piece.isVisible();
+        if (groupPiecesLevel [i] != null) {
+          if (minLevel == null
+              || groupPiecesLevel [i].getElevation() < minLevel.getElevation()) {
+            minLevel = groupPiecesLevel [i];
+          }
+        }
+        i++;
       }
 
-      String furnitureGroupName = this.preferences.getLocalizedString(
-          FurnitureController.class, "groupName", getFurnitureGroupCount(homeFurniture) + 1);
-      final HomeFurnitureGroup furnitureGroup = new HomeFurnitureGroup(selectedFurniture, furnitureGroupName);
+      final HomeFurnitureGroup furnitureGroup = createHomeFurnitureGroup(selectedFurniture);
+      final float [] groupPiecesNewElevation = new float [groupPieces.length];
+      i = 0;
+      for (HomePieceOfFurniture piece : sortedMap.values()) {
+        groupPiecesNewElevation [i++] = piece.getElevation();
+      }
       final int furnitureGroupIndex = homeFurniture.size() - groupPieces.length;
       final boolean movable = furnitureGroup.isMovable();
+      final Level groupLevel = minLevel;
       
       doGroupFurniture(groupPieces, new HomeFurnitureGroup [] {furnitureGroup}, 
-          new int [] {furnitureGroupIndex}, basePlanLocked);
+          new int [] {furnitureGroupIndex}, new Level [] {groupLevel}, basePlanLocked);
       if (this.undoSupport != null) {
         UndoableEdit undoableEdit = new AbstractUndoableEdit() {
             @Override
             public void undo() throws CannotUndoException {
               super.undo();
               doUngroupFurniture(groupPieces, groupPiecesIndex, 
-                  new HomeFurnitureGroup [] {furnitureGroup}, basePlanLocked);
+                  new HomeFurnitureGroup [] {furnitureGroup}, groupPiecesLevel, basePlanLocked);
               for (int i = 0; i < groupPieces.length; i++) {
+                groupPieces [i].setElevation(groupPiecesElevation [i]);
                 groupPieces [i].setMovable(groupPiecesMovable [i]);
                 groupPieces [i].setVisible(groupPiecesVisible [i]);
               }
@@ -489,10 +546,14 @@ public class FurnitureController implements Controller {
             @Override
             public void redo() throws CannotRedoException {
               super.redo();
-              doGroupFurniture(groupPieces, new HomeFurnitureGroup [] {furnitureGroup}, 
-                  new int [] {furnitureGroupIndex}, basePlanLocked);
+              for (int i = 0; i < groupPieces.length; i++) {
+                groupPieces [i].setElevation(groupPiecesNewElevation [i]);
+                groupPieces [i].setLevel(null);
+              }
               furnitureGroup.setMovable(movable);
               furnitureGroup.setVisible(true);
+              doGroupFurniture(groupPieces, new HomeFurnitureGroup [] {furnitureGroup}, 
+                  new int [] {furnitureGroupIndex}, new Level [] {groupLevel}, basePlanLocked);
             }
             
             @Override
@@ -505,6 +566,16 @@ public class FurnitureController implements Controller {
     }
   }
 
+  /**
+   * Returns a new furniture group for the given furniture list.
+   */
+  protected HomeFurnitureGroup createHomeFurnitureGroup(List<HomePieceOfFurniture> furniture) {
+    String furnitureGroupName = this.preferences.getLocalizedString(
+        FurnitureController.class, "groupName", getFurnitureGroupCount(this.home.getFurniture()) + 1);
+    final HomeFurnitureGroup furnitureGroup = new HomeFurnitureGroup(furniture, furnitureGroupName);
+    return furnitureGroup;
+  }
+
   /**
    * Returns the count of furniture groups among the given list.
    */
@@ -521,17 +592,19 @@ public class FurnitureController implements Controller {
   private void doGroupFurniture(HomePieceOfFurniture [] groupPieces,
                                 HomeFurnitureGroup [] furnitureGroups,
                                 int [] furnitureGroupsIndex,
+                                Level [] groupLevels, 
                                 boolean basePlanLocked) {
     doDeleteFurniture(groupPieces, basePlanLocked);
-    doAddFurniture(furnitureGroups, furnitureGroupsIndex, basePlanLocked);
+    doAddFurniture(furnitureGroups, furnitureGroupsIndex, null, groupLevels, basePlanLocked);
   }
 
   private void doUngroupFurniture(HomePieceOfFurniture [] groupPieces,
                                   int [] groupPiecesIndex,
                                   HomeFurnitureGroup [] furnitureGroups,
+                                  Level [] groupLevels, 
                                   boolean basePlanLocked) {
     doDeleteFurniture(furnitureGroups, basePlanLocked);
-    doAddFurniture(groupPieces, groupPiecesIndex, basePlanLocked);
+    doAddFurniture(groupPieces, groupPiecesIndex, null, groupLevels, basePlanLocked);
   }
 
   /**
@@ -572,29 +645,31 @@ public class FurnitureController implements Controller {
       final HomePieceOfFurniture [] groupPieces = 
           groupPiecesList.toArray(new HomePieceOfFurniture [groupPiecesList.size()]);      
       final int [] groupPiecesIndex = new int [groupPieces.length];
+      final Level [] groupPiecesLevel = new Level [groupPieces.length];
       int endIndex = homeFurniture.size() - furnitureGroups.length;
       boolean basePlanLocked = oldBasePlanLocked;
       for (i = 0; i < groupPieces.length; i++) {
         groupPiecesIndex [i] = endIndex++; 
+        groupPiecesLevel [i] = groupPieces [i].getLevel();
         // Unlock base plan if the piece is a part of it
         basePlanLocked &= !isPieceOfFurniturePartOfBasePlan(groupPieces [i]);
       }  
       final boolean newBasePlanLocked = basePlanLocked;
 
-      doUngroupFurniture(groupPieces, groupPiecesIndex, furnitureGroups, newBasePlanLocked);
+      doUngroupFurniture(groupPieces, groupPiecesIndex, furnitureGroups, groupPiecesLevel, newBasePlanLocked);
       if (this.undoSupport != null) {
         UndoableEdit undoableEdit = new AbstractUndoableEdit() {
             @Override
             public void undo() throws CannotUndoException {
               super.undo();
-              doGroupFurniture(groupPieces, furnitureGroups, furnitureGroupsIndex, oldBasePlanLocked);
+              doGroupFurniture(groupPieces, furnitureGroups, furnitureGroupsIndex, groupPiecesLevel, oldBasePlanLocked);
               home.setSelectedItems(oldSelection);
             }
             
             @Override
             public void redo() throws CannotRedoException {
               super.redo();
-              doUngroupFurniture(groupPieces, groupPiecesIndex, furnitureGroups, newBasePlanLocked);
+              doUngroupFurniture(groupPieces, groupPiecesIndex, furnitureGroups, groupPiecesLevel, newBasePlanLocked);
             }
             
             @Override
@@ -611,7 +686,7 @@ public class FurnitureController implements Controller {
    * Displays the wizard that helps to import furniture to home. 
    */
   public void importFurniture() {
-    new ImportedFurnitureWizardController(this.home, this.preferences, this.viewFactory, 
+    new ImportedFurnitureWizardController(this.home, this.preferences, this, this.viewFactory, 
         this.contentManager, this.undoSupport).displayView(getView());
   }
   
@@ -620,7 +695,7 @@ public class FurnitureController implements Controller {
    * given model name. 
    */
   public void importFurniture(String modelName) {
-    new ImportedFurnitureWizardController(this.home, modelName, this.preferences, 
+    new ImportedFurnitureWizardController(this.home, modelName, this.preferences, this, 
         this.viewFactory, this.contentManager, this.undoSupport).displayView(getView());
   }
   
