@@ -46,27 +46,24 @@ import javax.swing.undo.UndoableEditSupport;
 import com.eteks.sweethome3d.model.AspectRatio;
 import com.eteks.sweethome3d.model.BackgroundImage;
 import com.eteks.sweethome3d.model.Camera;
-import com.eteks.sweethome3d.model.CatalogDoorOrWindow;
-import com.eteks.sweethome3d.model.CatalogLight;
 import com.eteks.sweethome3d.model.CatalogPieceOfFurniture;
 import com.eteks.sweethome3d.model.CatalogTexture;
 import com.eteks.sweethome3d.model.CollectionEvent;
 import com.eteks.sweethome3d.model.CollectionListener;
 import com.eteks.sweethome3d.model.Compass;
 import com.eteks.sweethome3d.model.DimensionLine;
-import com.eteks.sweethome3d.model.DoorOrWindow;
+import com.eteks.sweethome3d.model.Elevatable;
 import com.eteks.sweethome3d.model.FurnitureCatalog;
 import com.eteks.sweethome3d.model.Home;
 import com.eteks.sweethome3d.model.HomeApplication;
 import com.eteks.sweethome3d.model.HomeDoorOrWindow;
 import com.eteks.sweethome3d.model.HomeEnvironment;
 import com.eteks.sweethome3d.model.HomeFurnitureGroup;
-import com.eteks.sweethome3d.model.HomeLight;
 import com.eteks.sweethome3d.model.HomePieceOfFurniture;
 import com.eteks.sweethome3d.model.HomeRecorder;
 import com.eteks.sweethome3d.model.InterruptedRecorderException;
 import com.eteks.sweethome3d.model.Label;
-import com.eteks.sweethome3d.model.Light;
+import com.eteks.sweethome3d.model.Level;
 import com.eteks.sweethome3d.model.RecorderException;
 import com.eteks.sweethome3d.model.Room;
 import com.eteks.sweethome3d.model.Selectable;
@@ -235,6 +232,7 @@ public class HomeController implements Controller {
     homeView.setEnabled(HomeView.ActionType.SORT_HOME_FURNITURE_BY_Y, true);
     homeView.setEnabled(HomeView.ActionType.SORT_HOME_FURNITURE_BY_ELEVATION, true);
     homeView.setEnabled(HomeView.ActionType.SORT_HOME_FURNITURE_BY_ANGLE, true);
+    homeView.setEnabled(HomeView.ActionType.SORT_HOME_FURNITURE_BY_LEVEL, true);
     homeView.setEnabled(HomeView.ActionType.SORT_HOME_FURNITURE_BY_COLOR, true);
     homeView.setEnabled(HomeView.ActionType.SORT_HOME_FURNITURE_BY_TEXTURE, true);
     homeView.setEnabled(HomeView.ActionType.SORT_HOME_FURNITURE_BY_MOVABILITY, true);
@@ -255,6 +253,7 @@ public class HomeController implements Controller {
     homeView.setEnabled(HomeView.ActionType.DISPLAY_HOME_FURNITURE_Y, true); 
     homeView.setEnabled(HomeView.ActionType.DISPLAY_HOME_FURNITURE_ELEVATION, true); 
     homeView.setEnabled(HomeView.ActionType.DISPLAY_HOME_FURNITURE_ANGLE, true); 
+    homeView.setEnabled(HomeView.ActionType.DISPLAY_HOME_FURNITURE_LEVEL, true); 
     homeView.setEnabled(HomeView.ActionType.DISPLAY_HOME_FURNITURE_COLOR, true); 
     homeView.setEnabled(HomeView.ActionType.DISPLAY_HOME_FURNITURE_TEXTURE, true); 
     homeView.setEnabled(HomeView.ActionType.DISPLAY_HOME_FURNITURE_MOVABLE, true); 
@@ -273,21 +272,23 @@ public class HomeController implements Controller {
     homeView.setEnabled(HomeView.ActionType.LOCK_BASE_PLAN, true);
     homeView.setEnabled(HomeView.ActionType.UNLOCK_BASE_PLAN, true);
     homeView.setEnabled(HomeView.ActionType.MODIFY_COMPASS, true);
-    homeView.setEnabled(HomeView.ActionType.IMPORT_BACKGROUND_IMAGE, true);
-    BackgroundImage backgroundImage = this.home.getBackgroundImage();
-    boolean homeHasBackgroundImage = backgroundImage != null;
-    homeView.setEnabled(HomeView.ActionType.MODIFY_BACKGROUND_IMAGE, homeHasBackgroundImage);
-    homeView.setEnabled(HomeView.ActionType.HIDE_BACKGROUND_IMAGE, 
-        homeHasBackgroundImage && backgroundImage.isVisible());
-    homeView.setEnabled(HomeView.ActionType.SHOW_BACKGROUND_IMAGE, 
-        homeHasBackgroundImage && !backgroundImage.isVisible());
-    homeView.setEnabled(HomeView.ActionType.DELETE_BACKGROUND_IMAGE, homeHasBackgroundImage);
+    Level selectedLevel = this.home.getSelectedLevel();
+    enableBackgroungImageActions(homeView, selectedLevel != null
+        ? selectedLevel.getBackgroundImage()
+        : this.home.getBackgroundImage());
+    homeView.setEnabled(HomeView.ActionType.ADD_LEVEL, true);
+    List<Level> levels = this.home.getLevels();
+    boolean homeContainsOneSelectedLevel = levels.size() > 1 && selectedLevel != null;
+    homeView.setEnabled(HomeView.ActionType.MODIFY_LEVEL, homeContainsOneSelectedLevel);
+    homeView.setEnabled(HomeView.ActionType.DELETE_LEVEL, homeContainsOneSelectedLevel);
     homeView.setEnabled(HomeView.ActionType.ZOOM_IN, true);
     homeView.setEnabled(HomeView.ActionType.ZOOM_OUT, true);
     homeView.setEnabled(HomeView.ActionType.EXPORT_TO_SVG, true); 
     homeView.setEnabled(HomeView.ActionType.VIEW_FROM_TOP, true);
     homeView.setEnabled(HomeView.ActionType.VIEW_FROM_OBSERVER, true);
     homeView.setEnabled(HomeView.ActionType.STORE_POINT_OF_VIEW, true);
+    homeView.setEnabled(HomeView.ActionType.DISPLAY_ALL_LEVELS, levels.size() > 1);
+    homeView.setEnabled(HomeView.ActionType.DISPLAY_SELECTED_LEVEL, levels.size() > 1);
     homeView.setEnabled(HomeView.ActionType.DETACH_3D_VIEW, true);
     homeView.setEnabled(HomeView.ActionType.ATTACH_3D_VIEW, true);
     homeView.setEnabled(HomeView.ActionType.VIEW_FROM_OBSERVER, true);
@@ -410,6 +411,7 @@ public class HomeController implements Controller {
     addFurnitureSortListener();
     addUndoSupportListener();
     addHomeItemsListener();
+    addLevelListeners();
     addPlanControllerListeners();
     addLanguageListener();
   }
@@ -600,19 +602,26 @@ public class HomeController implements Controller {
       this.home.addPropertyChangeListener(Home.Property.BACKGROUND_IMAGE, 
           new PropertyChangeListener() {
             public void propertyChange(PropertyChangeEvent ev) {
-              BackgroundImage backgroundImage = (BackgroundImage)ev.getNewValue();
-              boolean homeHasBackgroundImage = backgroundImage != null;
-              getView().setEnabled(HomeView.ActionType.MODIFY_BACKGROUND_IMAGE, homeHasBackgroundImage);
-              getView().setEnabled(HomeView.ActionType.HIDE_BACKGROUND_IMAGE, 
-                  homeHasBackgroundImage && backgroundImage.isVisible());
-              getView().setEnabled(HomeView.ActionType.SHOW_BACKGROUND_IMAGE, 
-                  homeHasBackgroundImage && !backgroundImage.isVisible());
-              getView().setEnabled(HomeView.ActionType.DELETE_BACKGROUND_IMAGE, homeHasBackgroundImage);
+              enableBackgroungImageActions(getView(), (BackgroundImage)ev.getNewValue());
             }
           });
     }
   }
 
+  /**
+   * Enables background image actions.
+   */
+  private void enableBackgroungImageActions(HomeView homeView, BackgroundImage backgroundImage) {
+    boolean homeHasBackgroundImage = backgroundImage != null;
+    getView().setEnabled(HomeView.ActionType.IMPORT_BACKGROUND_IMAGE, !homeHasBackgroundImage);
+    getView().setEnabled(HomeView.ActionType.MODIFY_BACKGROUND_IMAGE, homeHasBackgroundImage);
+    getView().setEnabled(HomeView.ActionType.HIDE_BACKGROUND_IMAGE, 
+        homeHasBackgroundImage && backgroundImage.isVisible());
+    getView().setEnabled(HomeView.ActionType.SHOW_BACKGROUND_IMAGE, 
+        homeHasBackgroundImage && !backgroundImage.isVisible());
+    getView().setEnabled(HomeView.ActionType.DELETE_BACKGROUND_IMAGE, homeHasBackgroundImage);
+  }
+
   /**
    * Adds listeners to track property changes that are not undoable.
    */
@@ -913,6 +922,61 @@ public class HomeController implements Controller {
       });
   }
 
+  /**
+   * Adds a property change listener to home to
+   * enable/disable authorized actions according to selected level.
+   */
+  private void addLevelListeners() {
+    final PropertyChangeListener selectedLevelListener = new PropertyChangeListener() {
+        public void propertyChange(PropertyChangeEvent ev) {
+          // Keep in selection only items that are at this level
+          List<Selectable> selectedItemsAtLevel = new ArrayList<Selectable>();
+          Level selectedLevel = home.getSelectedLevel();
+          for (Selectable item : home.getSelectedItems()) {
+            if (!(item instanceof Elevatable)
+                || ((Elevatable)item).isAtLevel(selectedLevel)) {
+              selectedItemsAtLevel.add(item);
+            }
+          }
+          home.setSelectedItems(selectedItemsAtLevel);
+          enableBackgroungImageActions(getView(), selectedLevel == null 
+              ? home.getBackgroundImage()
+              : selectedLevel.getBackgroundImage());
+          List<Level> levels = home.getLevels();
+          boolean homeContainsOneSelectedLevel = levels.size() > 1 && selectedLevel != null;
+          getView().setEnabled(HomeView.ActionType.MODIFY_LEVEL, homeContainsOneSelectedLevel);
+          getView().setEnabled(HomeView.ActionType.DELETE_LEVEL, homeContainsOneSelectedLevel);
+          getView().setEnabled(HomeView.ActionType.DISPLAY_ALL_LEVELS, levels.size() > 1);
+          getView().setEnabled(HomeView.ActionType.DISPLAY_SELECTED_LEVEL, levels.size() > 1);
+        }
+      };
+    this.home.addPropertyChangeListener(Home.Property.SELECTED_LEVEL, selectedLevelListener);
+    final PropertyChangeListener backgroundImageChangeListener = new PropertyChangeListener() {
+        public void propertyChange(PropertyChangeEvent ev) {
+          if (Level.Property.BACKGROUND_IMAGE.name().equals(ev.getPropertyName())) {
+            enableBackgroungImageActions(getView(), (BackgroundImage)ev.getNewValue());
+          }
+        }
+      };
+    for (Level level : home.getLevels()) {
+      level.addPropertyChangeListener(backgroundImageChangeListener);
+    }
+    this.home.addLevelsListener(new CollectionListener<Level>() {
+        public void collectionChanged(CollectionEvent<Level> ev) {
+          switch (ev.getType()) {
+            case ADD :
+              home.setSelectedLevel(ev.getItem());
+              ev.getItem().addPropertyChangeListener(backgroundImageChangeListener);
+              break;
+            case DELETE :
+              selectedLevelListener.propertyChange(null);
+              ev.getItem().removePropertyChangeListener(backgroundImageChangeListener);
+              break;
+          }
+        }
+      });
+  }
+
   /**
    * Adds a property change listener to plan controller to 
    * enable/disable authorized actions according to its modification state and the plan scale.
@@ -955,14 +1019,7 @@ public class HomeController implements Controller {
       List<HomePieceOfFurniture> newFurniture = 
           new ArrayList<HomePieceOfFurniture>();
       for (CatalogPieceOfFurniture piece : selectedFurniture) {
-        HomePieceOfFurniture homePiece;
-        if (piece instanceof CatalogDoorOrWindow) {
-          homePiece = new HomeDoorOrWindow((DoorOrWindow)piece);
-        } else if (piece instanceof CatalogLight) {
-          homePiece = new HomeLight((Light)piece);
-        } else {
-          homePiece = new HomePieceOfFurniture(piece);
-        }
+        HomePieceOfFurniture homePiece = getFurnitureController().createHomePieceOfFurniture(piece);
         // If magnetism is enabled, adjust piece size and elevation
         if (this.preferences.isMagnetismEnabled()) {
           if (homePiece.isResizable()) {
@@ -992,6 +1049,20 @@ public class HomeController implements Controller {
     }    
   }
   
+  /**
+   * Imports a language library chosen by the user.  
+   */
+  public void importLanguageLibrary() {
+    getView().invokeLater(new Runnable() {
+        public void run() {
+          final String languageLibraryName = getView().showImportLanguageLibraryDialog();
+          if (languageLibraryName != null) {
+            importLanguageLibrary(languageLibraryName);
+          }
+        }
+      });
+  }
+
   /**
    * Imports a given language library. 
    */
@@ -1184,7 +1255,7 @@ public class HomeController implements Controller {
         }
       }
     }
-    addItems(items, pastedItemsDelta, pastedItemsDelta, false, "undoPasteName");
+    addPastedItems(items, pastedItemsDelta, pastedItemsDelta, false, "undoPasteName");
   }
 
   /**
@@ -1200,15 +1271,15 @@ public class HomeController implements Controller {
    * and posts a drop operation to undo support.
    */
   public void drop(final List<? extends Selectable> items, View destinationView, float dx, float dy) {
-    addItems(items, dx, dy, destinationView == getPlanController().getView(), "undoDropName");
+    addPastedItems(items, dx, dy, destinationView == getPlanController().getView(), "undoDropName");
   }
 
   /**
    * Adds items to home.
    */
-  private void addItems(final List<? extends Selectable> items, 
-                        float dx, float dy, final boolean isDropInPlanView, 
-                        final String presentationNameKey) {
+  private void addPastedItems(final List<? extends Selectable> items, 
+                              float dx, float dy, final boolean isDropInPlanView, 
+                              final String presentationNameKey) {
     if (items.size() > 1
         || (items.size() == 1
             && !(items.get(0) instanceof Compass))) {
@@ -1223,7 +1294,10 @@ public class HomeController implements Controller {
         for (HomePieceOfFurniture piece : addedFurniture) {
           if (piece.isResizable()) {
             piece.setWidth(this.preferences.getLengthUnit().getMagnetizedLength(piece.getWidth(), 0.1f));
-            piece.setDepth(this.preferences.getLengthUnit().getMagnetizedLength(piece.getDepth(), 0.1f));
+            // Don't adjust depth of doors or windows otherwise they may be misplaced in a wall 
+            if (!(piece instanceof HomeDoorOrWindow) || dx != 0 || dy != 0) {
+              piece.setDepth(this.preferences.getLengthUnit().getMagnetizedLength(piece.getDepth(), 0.1f));
+            }
             piece.setHeight(this.preferences.getLengthUnit().getMagnetizedLength(piece.getHeight(), 0.1f));
           }
           piece.setElevation(this.preferences.getLengthUnit().getMagnetizedLength(piece.getElevation(), 0.1f));
@@ -1841,7 +1915,7 @@ public class HomeController implements Controller {
    */
   public void editPreferences() {
     new UserPreferencesController(this.preferences, 
-        this.viewFactory, this.contentManager).displayView(getView());
+        this.viewFactory, this.contentManager, this).displayView(getView());
   }
   
   /**
@@ -1911,25 +1985,28 @@ public class HomeController implements Controller {
    * Toggles visibility of the background image and posts an undoable operation.
    */
   private void toggleBackgroundImageVisibility(final String presentationName) {
+    final Level selectedLevel = this.home.getSelectedLevel();
     doToggleBackgroundImageVisibility(); 
     UndoableEdit undoableEdit = new AbstractUndoableEdit() {
-      @Override
-      public void undo() throws CannotUndoException {
-        super.undo();
-        doToggleBackgroundImageVisibility(); 
-      }
-      
-      @Override
-      public void redo() throws CannotRedoException {
-        super.redo();
-        doToggleBackgroundImageVisibility();
-      }
-      
-      @Override
-      public String getPresentationName() {
-        return preferences.getLocalizedString(HomeController.class, presentationName);
-      }
-    };
+        @Override
+        public void undo() throws CannotUndoException {
+          super.undo();
+          home.setSelectedLevel(selectedLevel);
+          doToggleBackgroundImageVisibility(); 
+        }
+        
+        @Override
+        public void redo() throws CannotRedoException {
+          super.redo();
+          home.setSelectedLevel(selectedLevel);
+          doToggleBackgroundImageVisibility();
+        }
+        
+        @Override
+        public String getPresentationName() {
+          return preferences.getLocalizedString(HomeController.class, presentationName);
+        }
+      };
     getUndoableEditSupport().postEdit(undoableEdit);
   }
 
@@ -1937,31 +2014,55 @@ public class HomeController implements Controller {
    * Toggles visibility of the background image.
    */
   private void doToggleBackgroundImageVisibility() {
-    BackgroundImage backgroundImage = this.home.getBackgroundImage();
-    this.home.setBackgroundImage(new BackgroundImage(backgroundImage.getImage(),
+    BackgroundImage backgroundImage = this.home.getSelectedLevel() != null
+        ? this.home.getSelectedLevel().getBackgroundImage()
+        : this.home.getBackgroundImage();
+    backgroundImage = new BackgroundImage(backgroundImage.getImage(),
         backgroundImage.getScaleDistance(), 
         backgroundImage.getScaleDistanceXStart(), backgroundImage.getScaleDistanceYStart(), 
         backgroundImage.getScaleDistanceXEnd(), backgroundImage.getScaleDistanceYEnd(),
-        backgroundImage.getXOrigin(), backgroundImage.getYOrigin(), !backgroundImage.isVisible()));
+        backgroundImage.getXOrigin(), backgroundImage.getYOrigin(), !backgroundImage.isVisible());
+    if (this.home.getSelectedLevel() != null) {
+      this.home.getSelectedLevel().setBackgroundImage(backgroundImage);
+    } else {
+      this.home.setBackgroundImage(backgroundImage);
+    }
   }
   
   /**
    * Deletes home background image and posts and posts an undoable operation. 
    */
   public void deleteBackgroundImage() {
-    final BackgroundImage oldImage = this.home.getBackgroundImage();
-    this.home.setBackgroundImage(null);
+    final Level selectedLevel = this.home.getSelectedLevel();
+    final BackgroundImage oldImage;
+    if (selectedLevel != null) {
+      oldImage = selectedLevel.getBackgroundImage();
+      selectedLevel.setBackgroundImage(null);
+    } else {
+      oldImage = this.home.getBackgroundImage();
+      this.home.setBackgroundImage(null);
+    }
     UndoableEdit undoableEdit = new AbstractUndoableEdit() {
       @Override
       public void undo() throws CannotUndoException {
         super.undo();
-        home.setBackgroundImage(oldImage); 
+        home.setSelectedLevel(selectedLevel);
+        if (selectedLevel != null) {
+          selectedLevel.setBackgroundImage(oldImage);
+        } else {
+          home.setBackgroundImage(oldImage);
+        }
       }
       
       @Override
       public void redo() throws CannotRedoException {
         super.redo();
-        home.setBackgroundImage(null);
+        home.setSelectedLevel(selectedLevel);
+        if (selectedLevel != null) {
+          selectedLevel.setBackgroundImage(null);
+        } else {
+          home.setBackgroundImage(null);
+        }
       }
       
       @Override
