@@ -33,8 +33,11 @@ import javax.swing.undo.UndoableEditSupport;
 import com.eteks.sweethome3d.model.Camera;
 import com.eteks.sweethome3d.model.CollectionEvent;
 import com.eteks.sweethome3d.model.CollectionListener;
+import com.eteks.sweethome3d.model.Elevatable;
 import com.eteks.sweethome3d.model.Home;
+import com.eteks.sweethome3d.model.HomeEnvironment;
 import com.eteks.sweethome3d.model.HomePieceOfFurniture;
+import com.eteks.sweethome3d.model.Level;
 import com.eteks.sweethome3d.model.ObserverCamera;
 import com.eteks.sweethome3d.model.Room;
 import com.eteks.sweethome3d.model.Selectable;
@@ -86,6 +89,23 @@ public class HomeController3D implements Controller {
               : observerCameraState);
         }
       });
+    // Add a listener to home to update visible levels according to selected level.
+    PropertyChangeListener selectedLevelListener = new PropertyChangeListener() {
+         public void propertyChange(PropertyChangeEvent ev) {
+           List<Level> levels = home.getLevels();
+           Level selectedLevel = home.getSelectedLevel();
+           boolean visible = true;
+           for (int i = 0; i < levels.size(); i++) {
+             levels.get(i).setVisible(visible);
+             if (levels.get(i) == selectedLevel
+                 && !home.getEnvironment().isAllLevelsVisible()) {
+               visible = false;
+             }
+           }
+         }
+       };
+     this.home.addPropertyChangeListener(Home.Property.SELECTED_LEVEL, selectedLevelListener);
+     this.home.getEnvironment().addPropertyChangeListener(HomeEnvironment.Property.ALL_LEVELS_VISIBLE, selectedLevelListener);
   }
 
   /**
@@ -138,8 +158,8 @@ public class HomeController3D implements Controller {
       }
     }
     storedCameras.add(0, camera);
-    // Ensure home stored cameras don't contain more than 10 cameras
-    while (storedCameras.size() > 10) {
+    // Ensure home stored cameras don't contain more than 20 cameras
+    while (storedCameras.size() > 20) {
       storedCameras.remove(storedCameras.size() - 1);
     }
     this.home.setStoredCameras(storedCameras);
@@ -162,6 +182,20 @@ public class HomeController3D implements Controller {
     this.home.setStoredCameras(storedCameras);
   }
   
+  /**
+   * Makes all levels visible.
+   */
+  public void displayAllLevels() {
+    this.home.getEnvironment().setAllLevelsVisible(true);
+  }
+  
+  /**
+   * Makes the selected level and below visible.
+   */
+  public void displaySelectedLevel() {
+    this.home.getEnvironment().setAllLevelsVisible(false);
+  }
+  
   /**
    * Controls the edition of 3D attributes. 
    */
@@ -260,12 +294,29 @@ public class HomeController3D implements Controller {
     private Camera      topCamera;
     private Rectangle2D homeBounds;
     private float       minDistanceToHomeCenter;
-    private PropertyChangeListener objectChangeListener = new PropertyChangeListener() {
-        public void propertyChange(PropertyChangeEvent evt) {
+    private PropertyChangeListener levelVisibilityChangeListener = new PropertyChangeListener() {
+        public void propertyChange(PropertyChangeEvent ev) {
+          if (Level.Property.VISIBLE.name().equals(ev.getPropertyName())) {
+            updateCameraFromHomeBounds();
+          }
+        }
+      };
+    private CollectionListener<Level> levelsListener = new CollectionListener<Level>() {
+        public void collectionChanged(CollectionEvent<Level> ev) {
+          if (ev.getType() == CollectionEvent.Type.ADD) {
+            ev.getItem().addPropertyChangeListener(levelVisibilityChangeListener);
+          } else if (ev.getType() == CollectionEvent.Type.DELETE) {
+            ev.getItem().removePropertyChangeListener(levelVisibilityChangeListener);
+          } 
           updateCameraFromHomeBounds();
         }
       };
-    private CollectionListener<Wall> wallListener = new CollectionListener<Wall>() {
+    private PropertyChangeListener objectChangeListener = new PropertyChangeListener() {
+        public void propertyChange(PropertyChangeEvent ev) {
+          updateCameraFromHomeBounds();
+        }
+      };
+    private CollectionListener<Wall> wallsListener = new CollectionListener<Wall>() {
         public void collectionChanged(CollectionEvent<Wall> ev) {
           if (ev.getType() == CollectionEvent.Type.ADD) {
             ev.getItem().addPropertyChangeListener(objectChangeListener);
@@ -300,10 +351,14 @@ public class HomeController3D implements Controller {
     public void enter() {
       this.topCamera = home.getCamera();
       updateCameraFromHomeBounds();
+      for (Level level : home.getLevels()) {
+        level.addPropertyChangeListener(this.levelVisibilityChangeListener);
+      }
+      home.addLevelsListener(this.levelsListener);
       for (Wall wall : home.getWalls()) {
         wall.addPropertyChangeListener(this.objectChangeListener);
       }
-      home.addWallsListener(this.wallListener);
+      home.addWallsListener(this.wallsListener);
       for (HomePieceOfFurniture piece : home.getFurniture()) {
         piece.addPropertyChangeListener(this.objectChangeListener);
       }
@@ -335,27 +390,32 @@ public class HomeController3D implements Controller {
     private Rectangle2D getHomeBounds() {
       // Compute plan bounds to include rooms, walls and furniture
       Rectangle2D homeBounds = null;
-      Collection<Wall> walls = home.getWalls();
-      for (Wall wall : walls) {
-        for (float [] point : wall.getPoints()) {
-          homeBounds = updateHomeBounds(homeBounds, point [0], point [1]);
+      boolean containsVisibleWalls = false;
+      for (Wall wall : home.getWalls()) {
+        if (isItemAtAVisibleLevel(wall)) {
+          containsVisibleWalls = true;
+          for (float [] point : wall.getPoints()) {
+            homeBounds = updateHomeBounds(homeBounds, point [0], point [1]);
+          }
         }
       }
       for (HomePieceOfFurniture piece : home.getFurniture()) {
-        if (piece.isVisible()) {
+        if (piece.isVisible() && isItemAtAVisibleLevel(piece)) {
           for (float [] point : piece.getPoints()) {
             homeBounds = updateHomeBounds(homeBounds, point [0], point [1]);
           }
         }
       }
       for (Room room : home.getRooms()) {
-        for (float [] point : room.getPoints()) {
-          homeBounds = updateHomeBounds(homeBounds, point [0], point [1]);
+        if (isItemAtAVisibleLevel(room)) {
+          for (float [] point : room.getPoints()) {
+            homeBounds = updateHomeBounds(homeBounds, point [0], point [1]);
+          }
         }
       }
 
       if (homeBounds != null) {
-        if (walls.isEmpty()) {
+        if (!containsVisibleWalls) {
           // If home contains only rooms and furniture don't fix minimum size
           return homeBounds;
         } else {
@@ -375,6 +435,13 @@ public class HomeController3D implements Controller {
       }
     }
 
+    /**
+     * Returns <code>true</code> if the given <code>item</code> is at a visible level.
+     */
+    private boolean isItemAtAVisibleLevel(Elevatable item) {
+      return item.getLevel() == null || item.getLevel().isVisible();
+    }
+    
     /**
      * Adds the point at the given coordinates to <code>homeBounds</code>.
      */
@@ -398,19 +465,23 @@ public class HomeController3D implements Controller {
         // If home contains no wall, search the max height of the highest piece
         for (HomePieceOfFurniture piece : home.getFurniture()) {
           if (piece.isVisible()) {
-            maxHeight = Math.max(maxHeight, piece.getElevation() + piece.getHeight());
+            maxHeight = Math.max(maxHeight, piece.getGroundElevation() + piece.getHeight());
           }
         }
       } else {
          // Search the max height of the highest wall
         for (Wall wall : walls) {
           Float height = wall.getHeight();
+          Level wallLevel = wall.getLevel();
+          float levelElevation = wallLevel == null 
+              ? 0
+              : wallLevel.getElevation(); 
           if (height != null) {
-            maxHeight = Math.max(maxHeight, height);
+            maxHeight = Math.max(maxHeight, levelElevation + height);
           }
           Float heightAtEnd = wall.getHeightAtEnd();
           if (heightAtEnd != null) {
-            maxHeight = Math.max(maxHeight, heightAtEnd);
+            maxHeight = Math.max(maxHeight, levelElevation + heightAtEnd);
           }
         }
       }
@@ -491,7 +562,7 @@ public class HomeController3D implements Controller {
       for (Wall wall : home.getWalls()) {
         wall.removePropertyChangeListener(this.objectChangeListener);
       }
-      home.removeWallsListener(wallListener);
+      home.removeWallsListener(wallsListener);
       for (HomePieceOfFurniture piece : home.getFurniture()) {
         piece.removePropertyChangeListener(this.objectChangeListener);
       }
@@ -500,6 +571,10 @@ public class HomeController3D implements Controller {
         room.removePropertyChangeListener(this.objectChangeListener);
       }
       home.removeRoomsListener(this.roomsListener);
+      for (Level room : home.getLevels()) {
+        room.removePropertyChangeListener(this.levelVisibilityChangeListener);
+      }
+      home.removeLevelsListener(this.levelsListener);
     }
   }
   
@@ -508,10 +583,31 @@ public class HomeController3D implements Controller {
    */
   private class ObserverCameraState extends CameraControllerState {
     private ObserverCamera observerCamera;
+    private PropertyChangeListener levelElevationChangeListener = new PropertyChangeListener() {
+        public void propertyChange(PropertyChangeEvent ev) {
+          if (Level.Property.ELEVATION.name().equals(ev.getPropertyName())) {
+            updateCameraMinimumElevation();
+          }
+        }
+      };
+    private CollectionListener<Level> levelsListener = new CollectionListener<Level>() {
+        public void collectionChanged(CollectionEvent<Level> ev) {
+          if (ev.getType() == CollectionEvent.Type.ADD) {
+            ev.getItem().addPropertyChangeListener(levelElevationChangeListener);
+          } else if (ev.getType() == CollectionEvent.Type.DELETE) {
+            ev.getItem().removePropertyChangeListener(levelElevationChangeListener);
+          } 
+          updateCameraMinimumElevation();
+        }
+      };
 
     @Override
     public void enter() {
       this.observerCamera = (ObserverCamera)home.getCamera();
+      for (Level level : home.getLevels()) {
+        level.addPropertyChangeListener(this.levelElevationChangeListener);
+      }
+      home.addLevelsListener(this.levelsListener);
       // Select observer camera for user feedback
       home.setSelectedItems(Arrays.asList(new Selectable [] {this.observerCamera}));
     }
@@ -527,13 +623,25 @@ public class HomeController3D implements Controller {
     @Override
     public void elevateCamera(float delta) {
       float newElevation = this.observerCamera.getZ() + delta; 
-      newElevation = Math.max(newElevation, 10 * 14 / 15);
-      newElevation = Math.min(newElevation, 1000 * 14 / 15);
+      newElevation = Math.min(Math.max(newElevation, getMinimumElevation()), preferences.getLengthUnit().getMaximumElevation());
       this.observerCamera.setZ(newElevation);
       // Select observer camera for user feedback
       home.setSelectedItems(Arrays.asList(new Selectable [] {this.observerCamera}));
     }
 
+    private void updateCameraMinimumElevation() {
+      observerCamera.setZ(Math.max(observerCamera.getZ(), getMinimumElevation()));
+    }
+
+    public float getMinimumElevation() {
+      List<Level> levels = home.getLevels();
+      if (levels.size() > 0) {
+        return 10 + levels.get(0).getElevation();
+      } else {
+        return 10;
+      }
+    }
+    
     @Override
     public void rotateCameraYaw(float delta) {
       this.observerCamera.setYaw(this.observerCamera.getYaw() + delta); 
@@ -568,6 +676,10 @@ public class HomeController3D implements Controller {
         selectedItems.remove(this.observerCamera);
         home.setSelectedItems(selectedItems);
       }
+      for (Level room : home.getLevels()) {
+        room.removePropertyChangeListener(this.levelElevationChangeListener);
+      }
+      home.removeLevelsListener(this.levelsListener);
       this.observerCamera = null;
     }
   }
