@@ -72,6 +72,7 @@ public class HomeCameraTest extends ComponentTestFixture {
     Locale.setDefault(Locale.FRANCE);
     UserPreferences preferences = new DefaultUserPreferences();
     Home home = new Home();
+    home.getCompass().setVisible(false);
     final HomeController controller = 
         new HomeController(home, preferences, new SwingViewFactory());
     JComponent homeView = (JComponent)controller.getView();
@@ -103,8 +104,8 @@ public class HomeCameraTest extends ComponentTestFixture {
         (float)Math.PI, (float)Math.PI / 4, home.getCamera());
     
     // 2. Create one wall between points (50, 50) and (150, 50) at a bigger scale
-    runAction(controller, HomePane.ActionType.CREATE_WALLS);
-    runAction(controller, HomePane.ActionType.ZOOM_IN);
+    runAction(controller, HomePane.ActionType.CREATE_WALLS, tester);
+    runAction(controller, HomePane.ActionType.ZOOM_IN, tester);
     tester.actionKeyPress(KeyEvent.VK_SHIFT);
     tester.actionClick(planComponent, 50, 50);
     tester.actionClick(planComponent, 150, 50, InputEvent.BUTTON1_MASK, 2);
@@ -127,37 +128,37 @@ public class HomeCameraTest extends ComponentTestFixture {
     // Add 1 to camera pitch
     tester.actionKeyStroke(KeyEvent.VK_PAGE_UP);
     // Check camera location and angles
-    assertCoordinatesAndAnglesEqualCameraLocationAndAngles(xWallMiddle, 1009.0617f, 1017.3002f, 
-        (float)Math.PI, (float)Math.PI / 4 + (float)Math.PI / 180, home.getCamera());
+    assertCoordinatesAndAnglesEqualCameraLocationAndAngles(xWallMiddle, 1000.1468f, 1025.8342f, 
+        (float)Math.PI, (float)Math.PI / 4 + (float)Math.PI / 120, home.getCamera());
     
     // 4. Remove 10 from camera yaw 
     tester.actionKeyStroke(KeyEvent.VK_LEFT);
     // Check camera location and angles
-    assertCoordinatesAndAnglesEqualCameraLocationAndAngles(263.9243f, 994.1371f, 1017.3002f, 
-        (float)Math.PI - 10 * (float)Math.PI / 180, (float)Math.PI / 4 + (float)Math.PI / 180, home.getCamera());
+    assertCoordinatesAndAnglesEqualCameraLocationAndAngles(144.2812f, 998.8128f, 1025.8342f, 
+        (float)Math.PI - (float)Math.PI / 60, (float)Math.PI / 4 + (float)Math.PI / 120, home.getCamera());
     // Add 1 to camera yaw 
     tester.actionKeyPress(KeyEvent.VK_SHIFT);
     tester.actionKeyStroke(KeyEvent.VK_RIGHT);
     tester.actionKeyRelease(KeyEvent.VK_SHIFT);
     // Check camera location and angles
-    assertCoordinatesAndAnglesEqualCameraLocationAndAngles(247.0138f, 996.967f, 1017.3002f, 
-        (float)Math.PI - 9 * (float)Math.PI / 180, (float)Math.PI / 4 + (float)Math.PI / 180, home.getCamera());
+    assertCoordinatesAndAnglesEqualCameraLocationAndAngles(-109.0647f, 978.874f, 1025.8342f, 
+        (float)Math.PI - (float)Math.PI / 60 + (float)Math.PI / 12, (float)Math.PI / 4 + (float)Math.PI / 120, home.getCamera());
     
     // 5. Move camera 1cm forward
     tester.actionKeyPress(KeyEvent.VK_SHIFT);
     tester.actionKeyStroke(KeyEvent.VK_UP);
     tester.actionKeyRelease(KeyEvent.VK_SHIFT);
     // Check camera location and angles
-    assertCoordinatesAndAnglesEqualCameraLocationAndAngles(246.4704f, 993.5365f, 1013.7034f, 
-        (float)Math.PI - 9 * (float)Math.PI / 180, (float)Math.PI / 4 + (float)Math.PI / 180, home.getCamera());
+    assertCoordinatesAndAnglesEqualCameraLocationAndAngles(-85.8082f, 869.4608f, 907.961f, 
+        (float)Math.PI - (float)Math.PI / 60 + (float)Math.PI / 12, (float)Math.PI / 4 + (float)Math.PI / 120, home.getCamera());
     // Move camera 10 backward 
     tester.actionKeyStroke(KeyEvent.VK_DOWN);
     // Check camera location and angles
-    assertCoordinatesAndAnglesEqualCameraLocationAndAngles(251.9039f, 1027.8417f, 1049.6704f, 
-        (float)Math.PI - 9 * (float)Math.PI / 180, (float)Math.PI / 4 + (float)Math.PI / 180, home.getCamera());
+    assertCoordinatesAndAnglesEqualCameraLocationAndAngles(-90.4595f, 891.3434f, 931.5356f, 
+        (float)Math.PI - (float)Math.PI / 60 + (float)Math.PI / 12, (float)Math.PI / 4 + (float)Math.PI / 120, home.getCamera());
     
     // 6. View from observer
-    runAction(controller, HomePane.ActionType.VIEW_FROM_OBSERVER);
+    runAction(controller, HomePane.ActionType.VIEW_FROM_OBSERVER, tester);
     tester.waitForIdle();
     ObserverCamera observerCamera = home.getObserverCamera();
     // Check camera is the observer camera
@@ -176,7 +177,7 @@ public class HomeCameraTest extends ComponentTestFixture {
     assertTrue("Camera isn't selected", home.getSelectedItems().contains(home.getCamera()));
 
     // Try to select wall and observer camera 
-    runAction(controller, HomePane.ActionType.SELECT);
+    runAction(controller, HomePane.ActionType.SELECT, tester);
     tester.actionClick(planComponent, 50, 50);
     tester.actionKeyPress(KeyEvent.VK_SHIFT);
     tester.actionClick(planComponent, (int)(140 * planComponent.getScale()), 
@@ -237,7 +238,7 @@ public class HomeCameraTest extends ComponentTestFixture {
     tester.actionKeyRelease(KeyEvent.VK_ALT);
     tester.actionMouseRelease();
     // Check camera location changed
-    assertCoordinatesAndAnglesEqualCameraLocationAndAngles(130.6284f, 141.8525f, 170, 
+    assertCoordinatesAndAnglesEqualCameraLocationAndAngles(108.657f, 111.4631f, 170, 
         2.5156f, 0.1639f, home.getCamera());
 
     // 10. Change observer camera yaw with mouse in 3D view
@@ -245,16 +246,16 @@ public class HomeCameraTest extends ComponentTestFixture {
     tester.actionMouseMove(component3D, new ComponentLocation(new Point(20, 20)));
     tester.actionMouseRelease();
     // Check camera yaw changed
-    assertCoordinatesAndAnglesEqualCameraLocationAndAngles(130.6284f, 141.8525f, 170, 
-        3.0156f, 0.1639f, home.getCamera());
+    assertCoordinatesAndAnglesEqualCameraLocationAndAngles(108.657f, 111.4631f, 170, 
+        2.5656f, 0.1639f, home.getCamera());
     
     // Change camera pitch with mouse in 3D view
     tester.actionMousePress(component3D, new ComponentLocation(new Point(20, 20)));
     tester.actionMouseMove(component3D, new ComponentLocation(new Point(20, 30)));
     tester.actionMouseRelease();
     // Check camera yaw changed
-    assertCoordinatesAndAnglesEqualCameraLocationAndAngles(130.6284f, 141.8525f, 170, 
-        3.0156f, 0.2139f, home.getCamera());
+    assertCoordinatesAndAnglesEqualCameraLocationAndAngles(108.657f, 111.4631f, 170, 
+        2.5656f, 0.2139f, home.getCamera());
     
     // 11. Edit 3D view modal dialog box
     JDialog attributesDialog = showHome3DAttributesPanel(preferences, controller, frame, tester);
@@ -312,12 +313,12 @@ public class HomeCameraTest extends ComponentTestFixture {
         0xFFFFFF, null, 0x000000, 0x808080, 1 / 255f * 128f, home);
     
     // 13. Undo changes
-    runAction(controller, HomePane.ActionType.UNDO);
+    runAction(controller, HomePane.ActionType.UNDO, tester);
     // Check home attributes have previous values
     assert3DAttributesEqualHomeAttributes(oldCameraFieldOfView, oldCameraHeight, 
         oldGroundColor, null, oldSkyColor, oldLightColor, oldWallsAlpha, home);
     // Redo
-    runAction(controller, HomePane.ActionType.REDO);
+    runAction(controller, HomePane.ActionType.REDO, tester);
     // Check home attributes are modified accordingly
     assert3DAttributesEqualHomeAttributes((float)Math.toRadians(90), 300f, 
         0xFFFFFF, null, 0x000000, 0x808080, 1 / 255f * 128f, home);
@@ -388,7 +389,7 @@ public class HomeCameraTest extends ComponentTestFixture {
     tester.invokeLater(new Runnable() { 
         public void run() {
           // Display dialog box later in Event Dispatch Thread to avoid blocking test thread
-          runAction(controller, HomePane.ActionType.MODIFY_3D_ATTRIBUTES);
+          ((JComponent)controller.getView()).getActionMap().get(HomePane.ActionType.MODIFY_3D_ATTRIBUTES).actionPerformed(null);
         }
       });
     // Wait for 3D view to be shown
@@ -419,11 +420,16 @@ public class HomeCameraTest extends ComponentTestFixture {
 
   /**
    * Runs <code>actionPerformed</code> method matching <code>actionType</code> 
-   * in <code>HomePane</code>. 
+   * in <code>controller</code> view. 
    */
-  private void runAction(HomeController controller,
-                         HomePane.ActionType actionType) {
-    ((JComponent)controller.getView()).getActionMap().get(actionType).actionPerformed(null);
+  private void runAction(final HomeController controller,
+                         final HomePane.ActionType actionType,
+                         JComponentTester tester) {
+    tester.invokeAndWait(new Runnable() { 
+        public void run() {
+          ((JComponent)controller.getView()).getActionMap().get(actionType).actionPerformed(null);
+        }
+      });
   }
 
   /**
